# -*- coding: utf-8 -*-

#
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogSimulator(object):
    def setupUi(self, DialogSimulator):
        DialogSimulator.setObjectName("DialogSimulator")
        DialogSimulator.resize(1088, 823)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(DialogSimulator)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.tabWidgetSimulatorSettings = QtWidgets.QTabWidget(DialogSimulator)
        self.tabWidgetSimulatorSettings.setStyleSheet("QTabWidget::pane { border: 0; }")
        self.tabWidgetSimulatorSettings.setObjectName("tabWidgetSimulatorSettings")
        self.tabLog = QtWidgets.QWidget()
        self.tabLog.setObjectName("tabLog")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tabLog)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.gvSimulator = LoggingGraphicsView(self.tabLog)
        self.gvSimulator.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.gvSimulator.setObjectName("gvSimulator")
        self.verticalLayout_3.addWidget(self.gvSimulator)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btnLogAll = QtWidgets.QPushButton(self.tabLog)
        self.btnLogAll.setAutoDefault(False)
        self.btnLogAll.setObjectName("btnLogAll")
        self.horizontalLayout.addWidget(self.btnLogAll)
        self.btnLogNone = QtWidgets.QPushButton(self.tabLog)
        self.btnLogNone.setAutoDefault(False)
        self.btnLogNone.setObjectName("btnLogNone")
        self.horizontalLayout.addWidget(self.btnLogNone)
        self.btnToggleLog = QtWidgets.QPushButton(self.tabLog)
        self.btnToggleLog.setAutoDefault(False)
        self.btnToggleLog.setObjectName("btnToggleLog")
        self.horizontalLayout.addWidget(self.btnToggleLog)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.tabWidgetSimulatorSettings.addTab(self.tabLog, "")
        self.tabRX = QtWidgets.QWidget()
        self.tabRX.setObjectName("tabRX")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.tabRX)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.scrollAreaRX = QtWidgets.QScrollArea(self.tabRX)
        self.scrollAreaRX.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollAreaRX.setWidgetResizable(True)
        self.scrollAreaRX.setObjectName("scrollAreaRX")
        self.scrollAreaWidgetContentsRX = QtWidgets.QWidget()
        self.scrollAreaWidgetContentsRX.setGeometry(QtCore.QRect(0, 0, 1066, 766))
        self.scrollAreaWidgetContentsRX.setObjectName("scrollAreaWidgetContentsRX")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContentsRX)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.btnTestSniffSettings = QtWidgets.QPushButton(self.scrollAreaWidgetContentsRX)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/sniffer.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnTestSniffSettings.setIcon(icon)
        self.btnTestSniffSettings.setAutoDefault(False)
        self.btnTestSniffSettings.setObjectName("btnTestSniffSettings")
        self.verticalLayout_6.addWidget(self.btnTestSniffSettings)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_6.addItem(spacerItem)
        self.scrollAreaRX.setWidget(self.scrollAreaWidgetContentsRX)
        self.verticalLayout_5.addWidget(self.scrollAreaRX)
        self.tabWidgetSimulatorSettings.addTab(self.tabRX, "")
        self.tabTX = QtWidgets.QWidget()
        self.tabTX.setObjectName("tabTX")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.tabTX)
        self.verticalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.scrollAreaTX = QtWidgets.QScrollArea(self.tabTX)
        self.scrollAreaTX.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollAreaTX.setWidgetResizable(True)
        self.scrollAreaTX.setObjectName("scrollAreaTX")
        self.scrollAreaWidgetContentsTX = QtWidgets.QWidget()
        self.scrollAreaWidgetContentsTX.setGeometry(QtCore.QRect(0, 0, 1066, 766))
        self.scrollAreaWidgetContentsTX.setObjectName("scrollAreaWidgetContentsTX")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContentsTX)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem1)
        self.scrollAreaTX.setWidget(self.scrollAreaWidgetContentsTX)
        self.verticalLayout_7.addWidget(self.scrollAreaTX)
        self.tabWidgetSimulatorSettings.addTab(self.tabTX, "")
        self.tabSimulation = QtWidgets.QWidget()
        self.tabSimulation.setObjectName("tabSimulation")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.tabSimulation)
        self.verticalLayout_9.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.tabWidget = QtWidgets.QTabWidget(self.tabSimulation)
        self.tabWidget.setStyleSheet("QTabWidget::pane { border: 0; }")
        self.tabWidget.setTabPosition(QtWidgets.QTabWidget.West)
        self.tabWidget.setTabShape(QtWidgets.QTabWidget.Triangular)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_simulation = QtWidgets.QWidget()
        self.tab_simulation.setObjectName("tab_simulation")
        self.verticalLayout_15 = QtWidgets.QVBoxLayout(self.tab_simulation)
        self.verticalLayout_15.setObjectName("verticalLayout_15")
        self.groupBoxSimulationStatus = QtWidgets.QGroupBox(self.tab_simulation)
        self.groupBoxSimulationStatus.setStyleSheet("QGroupBox\n"
"{\n"
"border: none;\n"
"}\n"
"\n"
"QGroupBox::title {\n"
"    subcontrol-origin: margin;\n"
"}\n"
"QGroupBox::indicator:unchecked {\n"
" image: url(:/icons/icons/collapse.svg)\n"
"}\n"
"QGroupBox::indicator:checked {\n"
" image: url(:/icons/icons/uncollapse.svg)\n"
"}")
        self.groupBoxSimulationStatus.setCheckable(True)
        self.groupBoxSimulationStatus.setObjectName("groupBoxSimulationStatus")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.groupBoxSimulationStatus)
        self.verticalLayout_12.setContentsMargins(-1, 15, -1, -1)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.frame = QtWidgets.QFrame(self.groupBoxSimulationStatus)
        self.frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame.setLineWidth(0)
        self.frame.setObjectName("frame")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.textEditSimulation = QtWidgets.QTextEdit(self.frame)
        self.textEditSimulation.setReadOnly(True)
        self.textEditSimulation.setObjectName("textEditSimulation")
        self.verticalLayout_11.addWidget(self.textEditSimulation)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(self.frame)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.lblCurrentRepeatValue = QtWidgets.QLabel(self.frame)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lblCurrentRepeatValue.setFont(font)
        self.lblCurrentRepeatValue.setAlignment(QtCore.Qt.AlignCenter)
        self.lblCurrentRepeatValue.setObjectName("lblCurrentRepeatValue")
        self.horizontalLayout_2.addWidget(self.lblCurrentRepeatValue)
        self.label_2 = QtWidgets.QLabel(self.frame)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.lblCurrentItemValue = QtWidgets.QLabel(self.frame)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lblCurrentItemValue.setFont(font)
        self.lblCurrentItemValue.setAlignment(QtCore.Qt.AlignCenter)
        self.lblCurrentItemValue.setObjectName("lblCurrentItemValue")
        self.horizontalLayout_2.addWidget(self.lblCurrentItemValue)
        self.btnSaveLog = QtWidgets.QToolButton(self.frame)
        icon = QtGui.QIcon.fromTheme("document-save")
        self.btnSaveLog.setIcon(icon)
        self.btnSaveLog.setObjectName("btnSaveLog")
        self.horizontalLayout_2.addWidget(self.btnSaveLog)
        self.verticalLayout_11.addLayout(self.horizontalLayout_2)
        self.verticalLayout_12.addWidget(self.frame)
        self.verticalLayout_15.addWidget(self.groupBoxSimulationStatus)
        self.groupBoxRXStatus = QtWidgets.QGroupBox(self.tab_simulation)
        self.groupBoxRXStatus.setStyleSheet("QGroupBox\n"
"{\n"
"border: none;\n"
"}\n"
"\n"
"QGroupBox::title {\n"
"    subcontrol-origin: margin;\n"
"}\n"
"QGroupBox::indicator:unchecked {\n"
" image: url(:/icons/icons/collapse.svg)\n"
"}\n"
"QGroupBox::indicator:checked {\n"
" image: url(:/icons/icons/uncollapse.svg)\n"
"}")
        self.groupBoxRXStatus.setCheckable(True)
        self.groupBoxRXStatus.setObjectName("groupBoxRXStatus")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout(self.groupBoxRXStatus)
        self.verticalLayout_14.setContentsMargins(-1, 15, -1, -1)
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.frame_2 = QtWidgets.QFrame(self.groupBoxRXStatus)
        self.frame_2.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_2.setLineWidth(0)
        self.frame_2.setObjectName("frame_2")
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(self.frame_2)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.checkBoxCaptureFullRX = QtWidgets.QCheckBox(self.frame_2)
        self.checkBoxCaptureFullRX.setObjectName("checkBoxCaptureFullRX")
        self.horizontalLayout_5.addWidget(self.checkBoxCaptureFullRX)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem2)
        self.btnSaveRX = QtWidgets.QToolButton(self.frame_2)
        icon = QtGui.QIcon.fromTheme("document-save")
        self.btnSaveRX.setIcon(icon)
        self.btnSaveRX.setObjectName("btnSaveRX")
        self.horizontalLayout_5.addWidget(self.btnSaveRX)
        self.verticalLayout_13.addLayout(self.horizontalLayout_5)
        self.graphicsViewPreview = LiveGraphicView(self.frame_2)
        self.graphicsViewPreview.setObjectName("graphicsViewPreview")
        self.verticalLayout_13.addWidget(self.graphicsViewPreview)
        self.verticalLayout_14.addWidget(self.frame_2)
        self.verticalLayout_15.addWidget(self.groupBoxRXStatus)
        self.tabWidget.addTab(self.tab_simulation, "")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout.setObjectName("verticalLayout")
        self.textEditTranscript = QtWidgets.QTextEdit(self.tab)
        self.textEditTranscript.setReadOnly(True)
        self.textEditTranscript.setObjectName("textEditTranscript")
        self.verticalLayout.addWidget(self.textEditTranscript)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.radioButtonTranscriptBit = QtWidgets.QRadioButton(self.tab)
        self.radioButtonTranscriptBit.setChecked(True)
        self.radioButtonTranscriptBit.setObjectName("radioButtonTranscriptBit")
        self.horizontalLayout_3.addWidget(self.radioButtonTranscriptBit)
        self.radioButtonTranscriptHex = QtWidgets.QRadioButton(self.tab)
        self.radioButtonTranscriptHex.setObjectName("radioButtonTranscriptHex")
        self.horizontalLayout_3.addWidget(self.radioButtonTranscriptHex)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem3)
        self.btnOpenInAnalysis = QtWidgets.QPushButton(self.tab)
        self.btnOpenInAnalysis.setObjectName("btnOpenInAnalysis")
        self.horizontalLayout_3.addWidget(self.btnOpenInAnalysis)
        self.btnSaveTranscript = QtWidgets.QToolButton(self.tab)
        icon = QtGui.QIcon.fromTheme("document-save")
        self.btnSaveTranscript.setIcon(icon)
        self.btnSaveTranscript.setObjectName("btnSaveTranscript")
        self.horizontalLayout_3.addWidget(self.btnSaveTranscript)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.tabWidget.addTab(self.tab, "")
        self.tab_device = QtWidgets.QWidget()
        self.tab_device.setObjectName("tab_device")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.tab_device)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.textEditDevices = QtWidgets.QTextEdit(self.tab_device)
        self.textEditDevices.setReadOnly(True)
        self.textEditDevices.setObjectName("textEditDevices")
        self.verticalLayout_10.addWidget(self.textEditDevices)
        self.tabWidget.addTab(self.tab_device, "")
        self.verticalLayout_9.addWidget(self.tabWidget)
        self.btnStartStop = QtWidgets.QPushButton(self.tabSimulation)
        icon = QtGui.QIcon.fromTheme("media-playback-start")
        self.btnStartStop.setIcon(icon)
        self.btnStartStop.setDefault(True)
        self.btnStartStop.setObjectName("btnStartStop")
        self.verticalLayout_9.addWidget(self.btnStartStop)
        self.tabWidgetSimulatorSettings.addTab(self.tabSimulation, "")
        self.verticalLayout_4.addWidget(self.tabWidgetSimulatorSettings)

        self.retranslateUi(DialogSimulator)
        self.tabWidgetSimulatorSettings.setCurrentIndex(3)
        self.tabWidget.setCurrentIndex(0)
        self.groupBoxSimulationStatus.toggled['bool'].connect(self.frame.setVisible)
        self.groupBoxRXStatus.toggled['bool'].connect(self.frame_2.setVisible)

    def retranslateUi(self, DialogSimulator):
        _translate = QtCore.QCoreApplication.translate
        DialogSimulator.setWindowTitle(_translate("DialogSimulator", "Simulation"))
        self.btnLogAll.setText(_translate("DialogSimulator", "Log all"))
        self.btnLogNone.setText(_translate("DialogSimulator", "Log none"))
        self.btnToggleLog.setText(_translate("DialogSimulator", "Toggle selected"))
        self.tabWidgetSimulatorSettings.setTabText(self.tabWidgetSimulatorSettings.indexOf(self.tabLog), _translate("DialogSimulator", "Log settings"))
        self.btnTestSniffSettings.setText(_translate("DialogSimulator", "Test sniffer settings"))
        self.tabWidgetSimulatorSettings.setTabText(self.tabWidgetSimulatorSettings.indexOf(self.tabRX), _translate("DialogSimulator", "RX settings"))
        self.tabWidgetSimulatorSettings.setTabText(self.tabWidgetSimulatorSettings.indexOf(self.tabTX), _translate("DialogSimulator", "TX settings"))
        self.groupBoxSimulationStatus.setTitle(_translate("DialogSimulator", "Simulation Status"))
        self.label.setText(_translate("DialogSimulator", "Current iteration:"))
        self.lblCurrentRepeatValue.setText(_translate("DialogSimulator", "0"))
        self.label_2.setText(_translate("DialogSimulator", "Current item:"))
        self.lblCurrentItemValue.setText(_translate("DialogSimulator", "0"))
        self.btnSaveLog.setText(_translate("DialogSimulator", "..."))
        self.groupBoxRXStatus.setTitle(_translate("DialogSimulator", "RX Status"))
        self.checkBoxCaptureFullRX.setText(_translate("DialogSimulator", "Capture complete RX"))
        self.btnSaveRX.setToolTip(_translate("DialogSimulator", "Save current capture"))
        self.btnSaveRX.setText(_translate("DialogSimulator", "Save"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_simulation), _translate("DialogSimulator", "Status"))
        self.textEditTranscript.setPlaceholderText(_translate("DialogSimulator", "Here you will find all messages that were sent and received during simulation."))
        self.radioButtonTranscriptBit.setText(_translate("DialogSimulator", "Bit &view"))
        self.radioButtonTranscriptHex.setText(_translate("DialogSimulator", "Hex view"))
        self.btnOpenInAnalysis.setText(_translate("DialogSimulator", "Open in Analysis"))
        self.btnSaveTranscript.setText(_translate("DialogSimulator", "..."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("DialogSimulator", "Messages"))
        self.textEditDevices.setPlaceholderText(_translate("DialogSimulator", "After simulation start you will see the log messages of your configured SDRs here."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_device), _translate("DialogSimulator", "Devices"))
        self.btnStartStop.setText(_translate("DialogSimulator", "Start"))
        self.tabWidgetSimulatorSettings.setTabText(self.tabWidgetSimulatorSettings.indexOf(self.tabSimulation), _translate("DialogSimulator", "Simulation"))
from urh.ui.views.LiveGraphicView import LiveGraphicView
from urh.ui.views.LoggingGraphicsView import LoggingGraphicsView
from . import urh_rc



import copy

from PyQt5.QtCore import Qt, pyqtSlot
from PyQt5.QtGui import QCloseEvent
from PyQt5.QtWidgets import QDialog

from urh import settings
from urh.models.RulesetTableModel import RulesetTableModel
from urh.signalprocessing import Ruleset
from urh.signalprocessing.MessageType import MessageType
from urh.signalprocessing.Ruleset import Rule, OPERATION_DESCRIPTION
from urh.ui.delegates.ComboBoxDelegate import ComboBoxDelegate
from urh.ui.ui_messagetype_options import Ui_DialogMessageType


class MessageTypeDialog(QDialog):

    def __init__(self, message_type: MessageType, parent=None):
        super().__init__(parent)
        self.ui = Ui_DialogMessageType()
        self.ui.setupUi(self)
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setWindowFlags(Qt.Window)

        operator_descriptions = list(OPERATION_DESCRIPTION.values())
        operator_descriptions.sort()

        self.setWindowTitle(self.tr("Rules for {}".format(message_type.name)))
        self.message_type = message_type
        self.original_ruleset = copy.deepcopy(message_type.ruleset)
        self.original_assigned_status = message_type.assigned_by_ruleset
        self.ruleset_table_model = RulesetTableModel(message_type.ruleset, operator_descriptions, parent=self)
        self.ui.tblViewRuleset.setModel(self.ruleset_table_model)

        self.ui.btnRemoveRule.setEnabled(len(message_type.ruleset) > 0)
        self.set_ruleset_ui_status()

        self.ui.rbAssignAutomatically.setChecked(self.message_type.assigned_by_ruleset)
        self.ui.rbAssignManually.setChecked(self.message_type.assign_manually)

        self.ui.tblViewRuleset.setItemDelegateForColumn(2, ComboBoxDelegate(["Bit", "Hex", "ASCII"], parent=self))
        self.ui.tblViewRuleset.setItemDelegateForColumn(3, ComboBoxDelegate(operator_descriptions, parent=self))

        for i in range(len(message_type.ruleset)):
            self.open_editors(i)

        self.ui.cbRulesetMode.setCurrentIndex(self.message_type.ruleset.mode.value)

        self.create_connects()
        self.restoreGeometry(settings.read("{}/geometry".format(self.__class__.__name__), type=bytes))

    def create_connects(self):
        self.ui.btnAddRule.clicked.connect(self.on_btn_add_rule_clicked)
        self.ui.btnRemoveRule.clicked.connect(self.on_btn_remove_rule_clicked)
        self.ui.rbAssignAutomatically.clicked.connect(self.on_rb_assign_automatically_clicked)
        self.ui.rbAssignManually.clicked.connect(self.on_rb_assign_manually_clicked)
        self.ui.cbRulesetMode.currentIndexChanged.connect(self.on_cb_rulesetmode_current_index_changed)

        self.ui.buttonBox.accepted.connect(self.accept)
        self.ui.buttonBox.rejected.connect(self.on_rejected)

    def set_ruleset_ui_status(self):
        self.ui.tblViewRuleset.setEnabled(self.message_type.assigned_by_ruleset)
        self.ui.btnRemoveRule.setEnabled(self.message_type.assigned_by_ruleset and len(self.message_type.ruleset) > 0)
        self.ui.btnAddRule.setEnabled(self.message_type.assigned_by_ruleset)
        self.ui.cbRulesetMode.setEnabled(self.message_type.assigned_by_ruleset)

    def open_editors(self, row):
        self.ui.tblViewRuleset.openPersistentEditor(self.ruleset_table_model.index(row, 2))
        self.ui.tblViewRuleset.openPersistentEditor(self.ruleset_table_model.index(row, 3))

    def closeEvent(self, event: QCloseEvent):
        self.ui.tblViewRuleset.setItemDelegateForColumn(2, None)
        self.ui.tblViewRuleset.setItemDelegateForColumn(3, None)
        settings.write("{}/geometry".format(self.__class__.__name__), self.saveGeometry())
        super().closeEvent(event)

    @pyqtSlot()
    def on_rejected(self):
        self.message_type.ruleset = self.original_ruleset
        self.message_type.assigned_by_ruleset = self.original_assigned_status
        self.reject()

    @pyqtSlot()
    def on_btn_add_rule_clicked(self):
        self.ui.btnRemoveRule.setEnabled(True)
        self.message_type.ruleset.append(Rule(start=0, end=0, operator="=", target_value="1", value_type=0))
        self.ruleset_table_model.update()

        for i in range(len(self.message_type.ruleset)):
            self.open_editors(i)

    @pyqtSlot()
    def on_btn_remove_rule_clicked(self):
        self.ruleset_table_model.ruleset.remove(self.message_type.ruleset[-1])
        self.ruleset_table_model.update()
        self.ui.btnRemoveRule.setEnabled(len(self.message_type.ruleset) > 0)

    @pyqtSlot()
    def on_rb_assign_automatically_clicked(self):
        self.message_type.assigned_by_ruleset = True
        self.set_ruleset_ui_status()

    @pyqtSlot()
    def on_rb_assign_manually_clicked(self):
        self.message_type.assigned_by_ruleset = False
        self.set_ruleset_ui_status()

    @pyqtSlot(int)
    def on_cb_rulesetmode_current_index_changed(self, index: int):
        self.message_type.ruleset.mode = Ruleset.Mode(index)



import os

from PyQt5.QtCore import QRegExp, Qt
from PyQt5.QtCore import pyqtSlot
from PyQt5.QtGui import QRegExpValidator, QCloseEvent
from PyQt5.QtWidgets import QDialog, QCompleter, QDirModel

from urh import settings
from urh.controller.dialogs.SpectrumDialogController import SpectrumDialogController
from urh.dev import config
from urh.models.ParticipantTableModel import ParticipantTableModel
from urh.signalprocessing.Participant import Participant
from urh.ui.ui_project import Ui_ProjectDialog
from urh.util import FileOperator
from urh.util.Errors import Errors
from urh.util.ProjectManager import ProjectManager


class ProjectDialog(QDialog):
    def __init__(self, new_project=True, project_manager: ProjectManager = None, parent=None):
        super().__init__(parent)
        if not new_project:
            assert project_manager is not None

        self.ui = Ui_ProjectDialog()
        self.ui.setupUi(self)
        self.setWindowFlags(Qt.Window)

        if new_project:
            self.participant_table_model = ParticipantTableModel([])
        else:
            self.participant_table_model = ParticipantTableModel(project_manager.participants)

            self.ui.spinBoxSampleRate.setValue(project_manager.device_conf["sample_rate"])
            self.ui.spinBoxFreq.setValue(project_manager.device_conf["frequency"])
            self.ui.spinBoxBandwidth.setValue(project_manager.device_conf["bandwidth"])
            self.ui.spinBoxGain.setValue(project_manager.device_conf.get("gain", config.DEFAULT_GAIN))
            self.ui.txtEdDescription.setPlainText(project_manager.description)
            self.ui.lineEdit_Path.setText(project_manager.project_path)
            self.ui.lineEditBroadcastAddress.setText(project_manager.broadcast_address_hex)

            self.ui.btnSelectPath.hide()
            self.ui.lineEdit_Path.setDisabled(True)
            self.setWindowTitle("Edit project settings")
            self.ui.lNewProject.setText("Edit project")

        self.ui.tblParticipants.setModel(self.participant_table_model)
        self.participant_table_model.update()

        self.ui.lineEditBroadcastAddress.setValidator(QRegExpValidator(QRegExp("([a-fA-F ]|[0-9]){,}")))

        self.sample_rate = self.ui.spinBoxSampleRate.value()
        self.freq = self.ui.spinBoxFreq.value()
        self.bandwidth = self.ui.spinBoxBandwidth.value()
        self.gain = self.ui.spinBoxGain.value()
        self.description = self.ui.txtEdDescription.toPlainText()
        self.broadcast_address_hex = self.ui.lineEditBroadcastAddress.text()

        self.path = self.ui.lineEdit_Path.text()
        self.new_project = new_project
        self.committed = False
        self.setModal(True)

        completer = QCompleter()
        completer.setModel(QDirModel(completer))
        self.ui.lineEdit_Path.setCompleter(completer)

        self.create_connects()
        # add two participants
        if self.participant_table_model.rowCount() == 0 and new_project:
            self.ui.btnAddParticipant.click()
            self.ui.btnAddParticipant.click()

        if new_project:
            self.ui.lineEdit_Path.setText(os.path.realpath(os.path.join(os.curdir, "new")))

        self.on_line_edit_path_text_edited()
        self.restoreGeometry(settings.read("{}/geometry".format(self.__class__.__name__), type=bytes))

    @property
    def participants(self):
        """

        :rtype: list of Participant
        """
        return self.participant_table_model.participants

    def create_connects(self):
        self.ui.spinBoxFreq.valueChanged.connect(self.on_spin_box_frequency_value_changed)
        self.ui.spinBoxSampleRate.valueChanged.connect(self.on_spin_box_sample_rate_value_changed)
        self.ui.spinBoxBandwidth.valueChanged.connect(self.on_spin_box_bandwidth_value_changed)
        self.ui.spinBoxGain.valueChanged.connect(self.on_spin_box_gain_value_changed)
        self.ui.txtEdDescription.textChanged.connect(self.on_txt_edit_description_text_changed)
        self.ui.lineEditBroadcastAddress.textEdited.connect(self.on_line_edit_broadcast_address_text_edited)

        self.ui.btnAddParticipant.clicked.connect(self.ui.tblParticipants.on_add_action_triggered)
        self.ui.btnRemoveParticipant.clicked.connect(self.ui.tblParticipants.on_remove_action_triggered)
        self.ui.btnUp.clicked.connect(self.ui.tblParticipants.on_move_up_action_triggered)
        self.ui.btnDown.clicked.connect(self.ui.tblParticipants.on_move_down_action_triggered)

        self.ui.lineEdit_Path.textEdited.connect(self.on_line_edit_path_text_edited)
        self.ui.buttonBox.accepted.connect(self.on_button_box_accepted)
        self.ui.buttonBox.rejected.connect(self.reject)
        self.ui.btnSelectPath.clicked.connect(self.on_btn_select_path_clicked)
        self.ui.lOpenSpectrumAnalyzer.linkActivated.connect(self.on_spectrum_analyzer_link_activated)

    def set_path(self, path):
        self.path = path
        self.ui.lineEdit_Path.setText(self.path)
        name = os.path.basename(os.path.normpath(self.path))
        self.ui.lblName.setText(name)

        self.ui.lblNewPath.setVisible(not os.path.isdir(self.path))

    def closeEvent(self, event: QCloseEvent):
        settings.write("{}/geometry".format(self.__class__.__name__), self.saveGeometry())
        super().closeEvent(event)

    @pyqtSlot(float)
    def on_spin_box_sample_rate_value_changed(self, value: float):
        self.sample_rate = value

    @pyqtSlot(float)
    def on_spin_box_frequency_value_changed(self, value: float):
        self.freq = value

    @pyqtSlot(float)
    def on_spin_box_bandwidth_value_changed(self, value: float):
        self.bandwidth = value

    @pyqtSlot(int)
    def on_spin_box_gain_value_changed(self, value: int):
        self.gain = value

    @pyqtSlot()
    def on_line_edit_path_text_edited(self):
        self.set_path(self.ui.lineEdit_Path.text())

    @pyqtSlot()
    def on_txt_edit_description_text_changed(self):
        self.description = self.ui.txtEdDescription.toPlainText()

    @pyqtSlot()
    def on_button_box_accepted(self):
        self.path = os.path.realpath(self.path)
        if not os.path.exists(self.path):
            try:
                os.makedirs(self.path)
            except Exception:
                pass

        # Path should be created now, if not raise Error
        if not os.path.exists(self.path):
            Errors.invalid_path(self.path)
            return

        self.committed = True
        self.accept()

    @pyqtSlot(str)
    def on_line_edit_broadcast_address_text_edited(self, value: str):
        self.broadcast_address_hex = value

    @pyqtSlot()
    def on_btn_select_path_clicked(self):
        directory = FileOperator.get_directory()
        if directory:
            self.set_path(directory)

    @pyqtSlot(dict)
    def set_recording_params_from_spectrum_analyzer_link(self, args: dict):
        self.ui.spinBoxFreq.setValue(args["frequency"])
        self.ui.spinBoxSampleRate.setValue(args["sample_rate"])
        self.ui.spinBoxBandwidth.setValue(args["bandwidth"])
        self.ui.spinBoxGain.setValue(args.get("gain", config.DEFAULT_GAIN))

    @pyqtSlot(str)
    def on_spectrum_analyzer_link_activated(self, link: str):
        if link == "open_spectrum_analyzer":
            r = SpectrumDialogController(ProjectManager(None), parent=self)
            if r.has_empty_device_list:
                Errors.no_device()
                r.close()
                return

            r.device_parameters_changed.connect(self.set_recording_params_from_spectrum_analyzer_link)
            r.show()



# -*- coding: utf-8 -*-

#
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ChecksumOptions(object):
    def setupUi(self, ChecksumOptions):
        ChecksumOptions.setObjectName("ChecksumOptions")
        ChecksumOptions.resize(775, 836)
        self.gridLayout = QtWidgets.QGridLayout(ChecksumOptions)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtWidgets.QScrollArea(ChecksumOptions)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 738, 827))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_3.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_4 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_3.addWidget(self.label_4)
        self.comboBoxCategory = QtWidgets.QComboBox(self.scrollAreaWidgetContents)
        self.comboBoxCategory.setObjectName("comboBoxCategory")
        self.comboBoxCategory.addItem("")
        self.comboBoxCategory.addItem("")
        self.verticalLayout_3.addWidget(self.comboBoxCategory)
        self.stackedWidget = QtWidgets.QStackedWidget(self.scrollAreaWidgetContents)
        self.stackedWidget.setObjectName("stackedWidget")
        self.page_crc = QtWidgets.QWidget()
        self.page_crc.setObjectName("page_crc")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.page_crc)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.lineEditFinalXOR = QtWidgets.QLineEdit(self.page_crc)
        self.lineEditFinalXOR.setObjectName("lineEditFinalXOR")
        self.gridLayout_2.addWidget(self.lineEditFinalXOR, 3, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.page_crc)
        self.label_5.setObjectName("label_5")
        self.gridLayout_2.addWidget(self.label_5, 1, 0, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(self.page_crc)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tableViewDataRanges = QtWidgets.QTableView(self.groupBox)
        self.tableViewDataRanges.setObjectName("tableViewDataRanges")
        self.tableViewDataRanges.horizontalHeader().setHighlightSections(False)
        self.tableViewDataRanges.verticalHeader().setCascadingSectionResizes(False)
        self.tableViewDataRanges.verticalHeader().setHighlightSections(False)
        self.horizontalLayout.addWidget(self.tableViewDataRanges)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.btnAddRange = QtWidgets.QToolButton(self.groupBox)
        icon = QtGui.QIcon.fromTheme("list-add")
        self.btnAddRange.setIcon(icon)
        self.btnAddRange.setObjectName("btnAddRange")
        self.verticalLayout.addWidget(self.btnAddRange)
        self.btnRemoveRange = QtWidgets.QToolButton(self.groupBox)
        icon = QtGui.QIcon.fromTheme("list-remove")
        self.btnRemoveRange.setIcon(icon)
        self.btnRemoveRange.setObjectName("btnRemoveRange")
        self.verticalLayout.addWidget(self.btnRemoveRange)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.gridLayout_2.addWidget(self.groupBox, 7, 0, 1, 2)
        self.lineEditStartValue = QtWidgets.QLineEdit(self.page_crc)
        self.lineEditStartValue.setObjectName("lineEditStartValue")
        self.gridLayout_2.addWidget(self.lineEditStartValue, 2, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.page_crc)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 0, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.page_crc)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 3, 0, 1, 1)
        self.label = QtWidgets.QLabel(self.page_crc)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 2, 0, 1, 1)
        self.lineEditCRCPolynomial = QtWidgets.QLineEdit(self.page_crc)
        self.lineEditCRCPolynomial.setObjectName("lineEditCRCPolynomial")
        self.gridLayout_2.addWidget(self.lineEditCRCPolynomial, 1, 1, 1, 1)
        self.comboBoxCRCFunction = QtWidgets.QComboBox(self.page_crc)
        self.comboBoxCRCFunction.setObjectName("comboBoxCRCFunction")
        self.gridLayout_2.addWidget(self.comboBoxCRCFunction, 0, 1, 1, 1)
        self.label_crc_info = QtWidgets.QLabel(self.page_crc)
        self.label_crc_info.setWordWrap(True)
        self.label_crc_info.setObjectName("label_crc_info")
        self.gridLayout_2.addWidget(self.label_crc_info, 6, 0, 1, 2)
        self.checkBoxRefIn = QtWidgets.QCheckBox(self.page_crc)
        self.checkBoxRefIn.setObjectName("checkBoxRefIn")
        self.gridLayout_2.addWidget(self.checkBoxRefIn, 4, 0, 1, 2)
        self.checkBoxRefOut = QtWidgets.QCheckBox(self.page_crc)
        self.checkBoxRefOut.setObjectName("checkBoxRefOut")
        self.gridLayout_2.addWidget(self.checkBoxRefOut, 5, 0, 1, 2)
        self.stackedWidget.addWidget(self.page_crc)
        self.page_wsp = QtWidgets.QWidget()
        self.page_wsp.setObjectName("page_wsp")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.page_wsp)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_6 = QtWidgets.QLabel(self.page_wsp)
        self.label_6.setAlignment(QtCore.Qt.AlignJustify|QtCore.Qt.AlignVCenter)
        self.label_6.setWordWrap(True)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_2.addWidget(self.label_6)
        self.radioButtonWSPAuto = QtWidgets.QRadioButton(self.page_wsp)
        self.radioButtonWSPAuto.setObjectName("radioButtonWSPAuto")
        self.verticalLayout_2.addWidget(self.radioButtonWSPAuto)
        self.radioButtonWSPChecksum4 = QtWidgets.QRadioButton(self.page_wsp)
        self.radioButtonWSPChecksum4.setObjectName("radioButtonWSPChecksum4")
        self.verticalLayout_2.addWidget(self.radioButtonWSPChecksum4)
        self.radioButtonWSPChecksum8 = QtWidgets.QRadioButton(self.page_wsp)
        self.radioButtonWSPChecksum8.setObjectName("radioButtonWSPChecksum8")
        self.verticalLayout_2.addWidget(self.radioButtonWSPChecksum8)
        self.radioButtonWSPCRC8 = QtWidgets.QRadioButton(self.page_wsp)
        self.radioButtonWSPCRC8.setObjectName("radioButtonWSPCRC8")
        self.verticalLayout_2.addWidget(self.radioButtonWSPCRC8)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.stackedWidget.addWidget(self.page_wsp)
        self.verticalLayout_3.addWidget(self.stackedWidget)
        spacerItem2 = QtWidgets.QSpacerItem(20, 107, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem2)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 1, 0, 1, 2)

        self.retranslateUi(ChecksumOptions)
        self.stackedWidget.setCurrentIndex(0)
        ChecksumOptions.setTabOrder(self.comboBoxCRCFunction, self.lineEditCRCPolynomial)
        ChecksumOptions.setTabOrder(self.lineEditCRCPolynomial, self.lineEditStartValue)
        ChecksumOptions.setTabOrder(self.lineEditStartValue, self.lineEditFinalXOR)
        ChecksumOptions.setTabOrder(self.lineEditFinalXOR, self.tableViewDataRanges)
        ChecksumOptions.setTabOrder(self.tableViewDataRanges, self.btnAddRange)
        ChecksumOptions.setTabOrder(self.btnAddRange, self.radioButtonWSPAuto)
        ChecksumOptions.setTabOrder(self.radioButtonWSPAuto, self.btnRemoveRange)
        ChecksumOptions.setTabOrder(self.btnRemoveRange, self.radioButtonWSPChecksum4)
        ChecksumOptions.setTabOrder(self.radioButtonWSPChecksum4, self.radioButtonWSPChecksum8)
        ChecksumOptions.setTabOrder(self.radioButtonWSPChecksum8, self.radioButtonWSPCRC8)

    def retranslateUi(self, ChecksumOptions):
        _translate = QtCore.QCoreApplication.translate
        ChecksumOptions.setWindowTitle(_translate("ChecksumOptions", "Configure Checksum"))
        self.label_4.setText(_translate("ChecksumOptions", "Checksum category:"))
        self.comboBoxCategory.setItemText(0, _translate("ChecksumOptions", "CRC"))
        self.comboBoxCategory.setItemText(1, _translate("ChecksumOptions", "Wireless Short Packet Checksum"))
        self.label_5.setText(_translate("ChecksumOptions", "CRC polynomial (hex):"))
        self.groupBox.setTitle(_translate("ChecksumOptions", "Configure data ranges for CRC"))
        self.btnAddRange.setText(_translate("ChecksumOptions", "..."))
        self.btnRemoveRange.setText(_translate("ChecksumOptions", "..."))
        self.label_3.setText(_translate("ChecksumOptions", "CRC function:"))
        self.label_2.setText(_translate("ChecksumOptions", "Final XOR (hex):"))
        self.label.setText(_translate("ChecksumOptions", "Start value (hex):"))
        self.label_crc_info.setText(_translate("ChecksumOptions", "<html><head/><body><p>Order=17</p><p>Length of checksum=16</p><p>start value length =16</p><p>final XOR length = 16</p><p>Polynomial = x<span style=\" vertical-align:super;\">1</span> + 4</p></body></html>"))
        self.checkBoxRefIn.setText(_translate("ChecksumOptions", "RefIn (Reflect input)"))
        self.checkBoxRefOut.setText(_translate("ChecksumOptions", "RefOut (Reflect output)"))
        self.label_6.setText(_translate("ChecksumOptions", "<html><head/><body><p>The Wireless Short Packet (WSP) standard uses three different checksums. URH can automatically detect the used checksum algorithm from the message. However, you can enforce the usage of a certain checksum if you need to.</p><p>With <span style=\" font-weight:600;\">Automatic</span> setting, checksums are chosen by these rules:</p><p>1) 4 Bit Checksum - For Switch Telegram (RORG=5 or 6 and STATUS = 0x20 or 0x30)</p><p>2) 8 Bit Checksum: STATUS bit 2<span style=\" vertical-align:super;\">7</span> = 0</p><p>3) 8 Bit CRC: STATUS bit 2<span style=\" vertical-align:super;\">7</span> = 1</p></body></html>"))
        self.radioButtonWSPAuto.setText(_translate("ChecksumOptions", "Automatic (recommended)"))
        self.radioButtonWSPChecksum4.setText(_translate("ChecksumOptions", "Force Checksum4"))
        self.radioButtonWSPChecksum8.setText(_translate("ChecksumOptions", "Force Checksum8"))
        self.radioButtonWSPCRC8.setText(_translate("ChecksumOptions", "Force CRC8"))



from optparse import OptionParser

import Initializer
from InputHandlerThread import InputHandlerThread

Initializer.init_path()

from gnuradio import blocks
from gnuradio import gr
import osmosdr


class top_block(gr.top_block):

    def __init__(self, sample_rate, frequency, freq_correction, rf_gain, if_gain, bb_gain, bandwidth, port):
        gr.top_block.__init__(self, "Top Block")

        self.sample_rate = sample_rate
        self.rf_gain = rf_gain
        self.port = port
        self.if_gain = if_gain
        self.frequency = frequency
        self.freq_correction = freq_correction
        self.bb_gain = bb_gain
        self.bandwidth = bandwidth

        self.osmosdr_sink_0 = osmosdr.sink(
            args="numchan=" + str(1) + " " + 'uhd'
        )
        self.osmosdr_sink_0.set_time_unknown_pps(osmosdr.time_spec_t())
        self.osmosdr_sink_0.set_sample_rate(sample_rate)
        self.osmosdr_sink_0.set_center_freq(frequency, 0)
        self.osmosdr_sink_0.set_freq_corr(freq_correction, 0)
        self.osmosdr_sink_0.set_gain(rf_gain, 0)
        self.osmosdr_sink_0.set_if_gain(if_gain, 0)
        self.osmosdr_sink_0.set_bb_gain(bb_gain, 0)
        self.osmosdr_sink_0.set_antenna('', 0)
        self.osmosdr_sink_0.set_bandwidth(bandwidth, 0)
        self.blocks_udp_source_0 = blocks.udp_source(gr.sizeof_gr_complex * 1, '127.0.0.1', port, 65536, False)

        self.connect((self.blocks_udp_source_0, 0), (self.osmosdr_sink_0, 0))

    def get_sample_rate(self):
        return self.sample_rate

    def set_sample_rate(self, sample_rate):
        self.sample_rate = sample_rate
        self.osmosdr_sink_0.set_sample_rate(self.sample_rate)

    def get_rf_gain(self):
        return self.rf_gain

    def set_rf_gain(self, rf_gain):
        self.rf_gain = rf_gain
        self.osmosdr_sink_0.set_gain(self.rf_gain, 0)

    def get_port(self):
        return self.port

    def set_port(self, port):
        self.port = port

    def get_if_gain(self):
        return self.if_gain

    def set_if_gain(self, if_gain):
        self.if_gain = if_gain
        self.osmosdr_sink_0.set_if_gain(self.if_gain, 0)

    def get_frequency(self):
        return self.frequency

    def set_frequency(self, frequency):
        self.frequency = frequency
        self.osmosdr_sink_0.set_center_freq(self.frequency, 0)

    def get_freq_correction(self):
        return self.freq_correction

    def set_freq_correction(self, freq_correction):
        self.freq_correction = freq_correction
        self.osmosdr_sink_0.set_freq_corr(self.freq_correction, 0)

    def get_direct_sampling_mode(self):
        return self.direct_sampling_mode

    def set_direct_sampling_mode(self, direct_sampling_mode):
        self.direct_sampling_mode = direct_sampling_mode

    def get_channel_index(self):
        return self.channel_index

    def set_channel_index(self, channel_index):
        self.channel_index = channel_index

    def get_bb_gain(self):
        return self.bb_gain

    def set_bb_gain(self, bb_gain):
        self.bb_gain = bb_gain
        self.osmosdr_sink_0.set_bb_gain(self.bb_gain, 0)

    def get_bandwidth(self):
        return self.bandwidth

    def set_bandwidth(self, bandwidth):
        self.bandwidth = bandwidth
        self.osmosdr_sink_0.set_bandwidth(self.bandwidth, 0)

    def get_antenna_index(self):
        return self.antenna_index

    def set_antenna_index(self, antenna_index):
        self.antenna_index = antenna_index


if __name__ == '__main__':
    parser = OptionParser(usage='%prog: [options]')
    parser.add_option('-s', '--sample-rate', dest='sample_rate', default=100000)
    parser.add_option('-f', '--frequency', dest='frequency', default=433000)
    parser.add_option('-g', '--gain', dest='rf_gain', default=30)
    parser.add_option('-i', '--if-gain', dest='if_gain', default=30)
    parser.add_option('-b', '--bb-gain', dest='bb_gain', default=30)
    parser.add_option('-w', '--bandwidth', dest='bandwidth', default=250000)
    parser.add_option('-c', '--freq-correction', dest='freq_correction', default=0)
    parser.add_option('-d', '--direct-sampling', dest='direct_sampling', default=0)
    parser.add_option('-n', '--channel-index', dest='channel_index', default=0)
    parser.add_option('-a', '--antenna-index', dest='antenna_index', default=0)
    parser.add_option('-p', '--port', dest='port', default=1234)

    (options, args) = parser.parse_args()
    tb = top_block(int(options.sample_rate), int(options.frequency), int(options.freq_correction), int(options.rf_gain),
                   int(options.if_gain), int(options.bb_gain), int(options.bandwidth), int(options.port))
    iht = InputHandlerThread(tb)
    iht.start()
    tb.start()
    tb.wait()



cimport urh.dev.native.lib.crtlsdr as crtlsdr
from libc.stdlib cimport malloc, free

ctypedef unsigned char uint8_t
ctypedef unsigned short uint16_t
ctypedef unsigned int  uint32_t
ctypedef unsigned long long uint64_t

cdef crtlsdr.rtlsdr_dev_t*_c_device

cdef void _c_callback_recv(unsigned char *buffer, uint32_t length, void *ctx):
    global f
    conn = <object> ctx
    (<object>f)(buffer[0:length])


IF RTLSDR_BANDWIDTH_SUPPORT == 1:
    cpdef bandwidth_is_adjustable():
        return True
    cpdef int set_tuner_bandwidth(uint32_t bw):
        """
        Set the bandwidth for the device.
    
        :param bw: bandwidth in Hz. Zero means automatic BW selection.
        :return 0 on success
        """
        return crtlsdr.rtlsdr_set_tuner_bandwidth(_c_device, bw)
ELSE:
    cpdef bandwidth_is_adjustable():
        return False
    cpdef int set_tuner_bandwidth(uint32_t bw):
        return -100

cpdef uint32_t get_device_count():
    return crtlsdr.rtlsdr_get_device_count()

cpdef str get_device_name(uint32_t index):
    return crtlsdr.rtlsdr_get_device_name(index).decode('UTF-8')

cpdef tuple get_device_usb_strings(uint32_t index):
    """

    :param index: index of the device
    :return: manufacturer name, product name, serial serial number on success else None, None, None
    """
    cdef char *manufacturer = <char *> malloc(256 * sizeof(char))
    cdef char *product = <char *> malloc(256 * sizeof(char))
    cdef char *serial = <char *> malloc(256 * sizeof(char))
    result = crtlsdr.rtlsdr_get_device_usb_strings(index, manufacturer, product, serial)
    try:
        if result == 0:
            return manufacturer.decode('UTF-8'), product.decode('UTF-8'), serial.decode('UTF-8')
        else:
            return None, None, None
    finally:
        free(manufacturer)
        free(product)
        free(serial)

cpdef int get_index_by_serial(str serial):
    """
    Get device index by USB serial string descriptor.

    :param serial: serial string of the device
    :return: device index of first device where the name matched
             -1 if name is NULL
             -2 if no devices were found at all
             -3 if devices were found, but none with matching name
    """
    serial_byte_string = serial.encode('UTF-8')
    return crtlsdr.rtlsdr_get_index_by_serial(<char *> serial_byte_string)

cpdef list get_device_list():
    result = []
    cdef uint32_t i, n = get_device_count()
    for i in range(n):
        manufacturer, product, serial = get_device_usb_strings(i)
        result.append("{} {} (SN: {})".format(manufacturer, product, serial))
    return result

cpdef int open(uint32_t index):
    return crtlsdr.rtlsdr_open(&_c_device, index)

cpdef int close():
    return crtlsdr.rtlsdr_close(_c_device)

cpdef int set_xtal_freq(uint32_t rtl_freq, uint32_t tuner_freq):
    """
     Set crystal oscillator frequencies used for the RTL2832 and the tuner IC.

    Usually both ICs use the same clock. Changing the clock may make sense if
    you are applying an external clock to the tuner or to compensate the
    frequency (and samplerate) error caused by the original (cheap) crystal.

    NOTE: Call this function only if you fully understand the implications.

    :param rtl_freq: frequency value used to clock the RTL2832 in Hz
    :param tuner_freq: frequency value used to clock the tuner IC in Hz
    :return: 0 on success
    """
    return crtlsdr.rtlsdr_set_xtal_freq(_c_device, rtl_freq, tuner_freq)

cpdef tuple get_xtal_freq():
    """
    Get crystal oscillator frequencies used for the RTL2832 and the tuner IC.
    Usually both ICs use the same clock.

    :return: rtl_freq frequency value used to clock the RTL2832 in Hz,
             tuner_freq frequency value used to clock the tuner IC in Hz
    """
    cdef uint32_t rtl_freq = 0
    cdef uint32_t tuner_freq = 0
    result = crtlsdr.rtlsdr_get_xtal_freq(_c_device, &rtl_freq, &tuner_freq)

    if result == 0:
        return rtl_freq, tuner_freq
    else:
        return None, None

cpdef tuple get_usb_strings():
    cdef char *manufacturer = <char *> malloc(256 * sizeof(char))
    cdef char *product = <char *> malloc(256 * sizeof(char))
    cdef char *serial = <char *> malloc(256 * sizeof(char))
    result = crtlsdr.rtlsdr_get_usb_strings(_c_device, manufacturer, product, serial)
    if result == 0:
        return manufacturer.decode('UTF-8'), product.decode('UTF-8'), serial.decode('UTF-8')
    else:
        return None, None, None

cpdef int set_center_freq(uint32_t freq):
    return crtlsdr.rtlsdr_set_center_freq(_c_device, freq)

cpdef uint32_t get_center_freq():
    """
    Get actual frequency the device is tuned to.

    :return: 0 on error, frequency in Hz otherwise
    """
    return crtlsdr.rtlsdr_get_center_freq(_c_device)

cpdef int set_freq_correction(int ppm):
    """
    Set the frequency correction value for the device.

    :param ppm: ppm correction value in parts per million (ppm)
    :return: 0 on success
    """
    return crtlsdr.rtlsdr_set_freq_correction(_c_device, ppm)

cpdef int get_freq_correction():
    """
    Get actual frequency correction value of the device.

    :return: correction value in parts per million (ppm)
    """
    return crtlsdr.rtlsdr_get_freq_correction(_c_device)

cpdef crtlsdr.rtlsdr_tuner get_tuner_type():
    """
    Get the tuner type.

    :return: RTLSDR_TUNER_UNKNOWN on error, tuner type otherwise
    """
    return crtlsdr.rtlsdr_get_tuner_type(_c_device)

cpdef list get_tuner_gains():
    """
    Get a list of gains supported by the tuner.
    NOTE: The gains argument must be preallocated by the caller. If NULL is
    being given instead, the number of available gain values will be returned.

    :return: gains array of gain values. In tenths of a dB, 115 means 11.5 dB.
    """
    cdef int num_gains = crtlsdr.rtlsdr_get_tuner_gains(_c_device, NULL)
    if num_gains < 0:
        return None

    cdef int*gains = <int *> malloc(num_gains * sizeof(int))
    crtlsdr.rtlsdr_get_tuner_gains(_c_device, gains)

    try:
        return [gains[i] for i in range(num_gains)]
    finally:
        free(gains)

cpdef int set_tuner_gain(int gain):
    """
    Set the gain for the device.
    Manual gain mode must be enabled for this to work.

    Valid gain values (in tenths of a dB) for the E4000 tuner:
    -10, 15, 40, 65, 90, 115, 140, 165, 190,
    215, 240, 290, 340, 420, 430, 450, 470, 490

    Valid gain values may be queried with rtlsdr_get_tuner_gains function.

    :param gain: gain in tenths of a dB, 115 means 11.5 dB.
    :return: 0 on success
    """
    return crtlsdr.rtlsdr_set_tuner_gain(_c_device, gain)

cpdef int get_tuner_gain():
    """
    Get actual gain the device is configured to.

    :return: 0 on error, gain in tenths of a dB, 115 means 11.5 dB.
    """
    return crtlsdr.rtlsdr_get_tuner_gain(_c_device)

cpdef int set_tuner_if_gain(int stage, int gain):
    """
    Set the intermediate frequency gain for the device.

    :param stage: intermediate frequency gain stage number (1 to 6 for E4000)
    :param gain: in tenths of a dB, -30 means -3.0 dB.
    :return: 0 on success
    """
    return crtlsdr.rtlsdr_set_tuner_if_gain(_c_device, stage, gain)

cpdef int set_tuner_gain_mode(int manual):
    """
    Set the gain mode (automatic/manual) for the device.
    Manual gain mode must be enabled for the gain setter function to work.

    :param manual: 1 means manual gain mode shall be enabled.
    :return: 0 on success
    """
    return crtlsdr.rtlsdr_set_tuner_gain_mode(_c_device, manual)

cpdef int set_sample_rate(uint32_t sample_rate):
    """
    Set the sample rate for the device, also selects the baseband filters
    according to the requested sample rate for tuners where this is possible.

    :param sample_rate: the sample rate to be set, possible values are:
                225001 - 300000 Hz
  		        900001 - 3200000 Hz
  		        sample loss is to be expected for rates > 2400000
    :return:
    """
    return crtlsdr.rtlsdr_set_sample_rate(_c_device, sample_rate)

cpdef uint32_t get_sample_rate():
    """
    Get actual sample rate the device is configured to.
    :return: 0 on error, sample rate in Hz otherwise
    """
    return crtlsdr.rtlsdr_get_sample_rate(_c_device)

cpdef int set_agc_mode(int on):
    """
    Enable or disable the internal digital Automatic Gain Control of the RTL2832.

    :param on: digital AGC mode, 1 means enabled, 0 disabled
    :return: 0 on success
    """
    return crtlsdr.rtlsdr_set_agc_mode(_c_device, on)

cpdef int set_direct_sampling(int on):
    """
    Enable or disable the direct sampling mode. When enabled, the IF mode
    of the RTL2832 is activated, and rtlsdr_set_center_freq() will control
    the IF-frequency of the DDC, which can be used to tune from 0 to 28.8 MHz
    (xtal frequency of the RTL2832).

    :param on: 0 means disabled, 1 I-ADC input enabled, 2 Q-ADC input enabled
    :return: 0 on success
    """
    return crtlsdr.rtlsdr_set_direct_sampling(_c_device, on)

cpdef int get_direct_sampling():
    """
    Get state of the direct sampling mode

    :return: -1 on error, 0 means disabled, 1 I-ADC input enabled, 2 Q-ADC input enabled
    """
    return crtlsdr.rtlsdr_get_direct_sampling(_c_device)

cpdef int set_offset_tuning(int on):
    """
    Enable or disable offset tuning for zero-IF tuners, which allows to avoid
    problems caused by the DC offset of the ADCs and 1/f noise.

    :param on: 0 means disabled, 1 enabled
    :return: 0 on success
    """
    return crtlsdr.rtlsdr_set_offset_tuning(_c_device, on)

cpdef int get_offset_tuning():
    """
    Get state of the offset tuning mode

    :return: -1 on error, 0 means disabled, 1 enabled
    """
    return crtlsdr.rtlsdr_get_offset_tuning(_c_device)

cpdef int reset_buffer():
    return crtlsdr.rtlsdr_reset_buffer(_c_device)

cpdef bytes read_sync(int num_samples=8 * 32 * 512):
    """
    The raw, captured IQ data is 8 bit unsigned data.

    :return:
    """
    cdef uint8_t *samples = <uint8_t *> malloc(2*num_samples * sizeof(uint8_t))
    if not samples:
        raise MemoryError()

    cdef int n_read = 0
    try:
        crtlsdr.rtlsdr_read_sync(_c_device, <void *>samples, num_samples, &n_read)
        return bytes(samples[0:n_read])
    finally:
        free(samples)

cpdef int read_async(callback, connection):
    """
    Read samples from the device asynchronously. This function will block until
    it is being canceled using rtlsdr_cancel_async()
    read_bytes_async
    :return: 0 on success
    """
    global f
    f = callback
    return crtlsdr.rtlsdr_read_async(_c_device, _c_callback_recv, <void *>connection, 0, 0)

cpdef int cancel_async():
    """
    Cancel all pending asynchronous operations on the device.

    :return: 0 on success
    """
    return crtlsdr.rtlsdr_cancel_async(_c_device)



# -*- coding: utf-8 -*-

#
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogModulation(object):
    def setupUi(self, DialogModulation):
        DialogModulation.setObjectName("DialogModulation")
        DialogModulation.resize(977, 1041)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/modulation.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        DialogModulation.setWindowIcon(icon)
        self.verticalLayout = QtWidgets.QVBoxLayout(DialogModulation)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout_5 = QtWidgets.QGridLayout()
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.comboBoxCustomModulations = QtWidgets.QComboBox(DialogModulation)
        self.comboBoxCustomModulations.setEditable(True)
        self.comboBoxCustomModulations.setInsertPolicy(QtWidgets.QComboBox.InsertAtCurrent)
        self.comboBoxCustomModulations.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.comboBoxCustomModulations.setObjectName("comboBoxCustomModulations")
        self.comboBoxCustomModulations.addItem("")
        self.gridLayout_5.addWidget(self.comboBoxCustomModulations, 0, 0, 1, 1)
        self.btnAddModulation = QtWidgets.QToolButton(DialogModulation)
        icon = QtGui.QIcon.fromTheme("list-add")
        self.btnAddModulation.setIcon(icon)
        self.btnAddModulation.setObjectName("btnAddModulation")
        self.gridLayout_5.addWidget(self.btnAddModulation, 0, 1, 1, 1)
        self.btnRemoveModulation = QtWidgets.QToolButton(DialogModulation)
        icon = QtGui.QIcon.fromTheme("list-remove")
        self.btnRemoveModulation.setIcon(icon)
        self.btnRemoveModulation.setObjectName("btnRemoveModulation")
        self.gridLayout_5.addWidget(self.btnRemoveModulation, 0, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout_5)
        self.scrollArea = QtWidgets.QScrollArea(DialogModulation)
        self.scrollArea.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents_2 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_2.setGeometry(QtCore.QRect(0, 0, 965, 984))
        self.scrollAreaWidgetContents_2.setObjectName("scrollAreaWidgetContents_2")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents_2)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.label_5 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.gridLayout_7.addWidget(self.label_5, 2, 0, 1, 1)
        self.lEqual = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lEqual.sizePolicy().hasHeightForWidth())
        self.lEqual.setSizePolicy(sizePolicy)
        self.lEqual.setMaximumSize(QtCore.QSize(32, 32))
        self.lEqual.setText("")
        self.lEqual.setPixmap(QtGui.QPixmap(":/icons/icons/equals.svg"))
        self.lEqual.setScaledContents(True)
        self.lEqual.setAlignment(QtCore.Qt.AlignCenter)
        self.lEqual.setObjectName("lEqual")
        self.gridLayout_7.addWidget(self.lEqual, 4, 2, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.gridLayout_7.addWidget(self.label_6, 4, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem, 8, 1, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.gridLayout_7.addWidget(self.label_7, 8, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem1, 2, 3, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem2, 4, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.gridLayout_7.addWidget(self.label_4, 0, 0, 1, 1)
        self.gVOriginalSignal = ZoomAndDropableGraphicView(self.scrollAreaWidgetContents_2)
        self.gVOriginalSignal.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.gVOriginalSignal.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.gVOriginalSignal.setRenderHints(QtGui.QPainter.Antialiasing|QtGui.QPainter.HighQualityAntialiasing)
        self.gVOriginalSignal.setDragMode(QtWidgets.QGraphicsView.NoDrag)
        self.gVOriginalSignal.setObjectName("gVOriginalSignal")
        self.gridLayout_7.addWidget(self.gVOriginalSignal, 9, 1, 1, 3)
        self.scrollArea_5 = QtWidgets.QScrollArea(self.scrollAreaWidgetContents_2)
        self.scrollArea_5.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea_5.setWidgetResizable(True)
        self.scrollArea_5.setObjectName("scrollArea_5")
        self.scrollAreaWidgetContents_5 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_5.setGeometry(QtCore.QRect(0, 0, 373, 330))
        self.scrollAreaWidgetContents_5.setObjectName("scrollAreaWidgetContents_5")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents_5)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.lCurrentSearchResult = QtWidgets.QLabel(self.scrollAreaWidgetContents_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lCurrentSearchResult.sizePolicy().hasHeightForWidth())
        self.lCurrentSearchResult.setSizePolicy(sizePolicy)
        self.lCurrentSearchResult.setMinimumSize(QtCore.QSize(0, 0))
        self.lCurrentSearchResult.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.lCurrentSearchResult.setAlignment(QtCore.Qt.AlignCenter)
        self.lCurrentSearchResult.setObjectName("lCurrentSearchResult")
        self.gridLayout_4.addWidget(self.lCurrentSearchResult, 3, 1, 1, 2)
        self.cbShowDataBitsOnly = QtWidgets.QCheckBox(self.scrollAreaWidgetContents_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cbShowDataBitsOnly.sizePolicy().hasHeightForWidth())
        self.cbShowDataBitsOnly.setSizePolicy(sizePolicy)
        self.cbShowDataBitsOnly.setMinimumSize(QtCore.QSize(0, 0))
        self.cbShowDataBitsOnly.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.cbShowDataBitsOnly.setObjectName("cbShowDataBitsOnly")
        self.gridLayout_4.addWidget(self.cbShowDataBitsOnly, 2, 0, 1, 5)
        self.btnSearchPrev = QtWidgets.QPushButton(self.scrollAreaWidgetContents_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnSearchPrev.sizePolicy().hasHeightForWidth())
        self.btnSearchPrev.setSizePolicy(sizePolicy)
        self.btnSearchPrev.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.btnSearchPrev.setText("")
        icon = QtGui.QIcon.fromTheme("go-previous")
        self.btnSearchPrev.setIcon(icon)
        self.btnSearchPrev.setObjectName("btnSearchPrev")
        self.gridLayout_4.addWidget(self.btnSearchPrev, 3, 0, 1, 1)
        self.lTotalSearchresults = QtWidgets.QLabel(self.scrollAreaWidgetContents_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lTotalSearchresults.sizePolicy().hasHeightForWidth())
        self.lTotalSearchresults.setSizePolicy(sizePolicy)
        self.lTotalSearchresults.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.lTotalSearchresults.setAlignment(QtCore.Qt.AlignCenter)
        self.lTotalSearchresults.setObjectName("lTotalSearchresults")
        self.gridLayout_4.addWidget(self.lTotalSearchresults, 3, 4, 1, 1)
        self.treeViewSignals = ModulatorTreeView(self.scrollAreaWidgetContents_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.treeViewSignals.sizePolicy().hasHeightForWidth())
        self.treeViewSignals.setSizePolicy(sizePolicy)
        self.treeViewSignals.setProperty("showDropIndicator", True)
        self.treeViewSignals.setDragEnabled(True)
        self.treeViewSignals.setDragDropMode(QtWidgets.QAbstractItemView.DragOnly)
        self.treeViewSignals.setHeaderHidden(True)
        self.treeViewSignals.setObjectName("treeViewSignals")
        self.gridLayout_4.addWidget(self.treeViewSignals, 0, 0, 1, 6)
        self.lSlash = QtWidgets.QLabel(self.scrollAreaWidgetContents_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSlash.sizePolicy().hasHeightForWidth())
        self.lSlash.setSizePolicy(sizePolicy)
        self.lSlash.setMaximumSize(QtCore.QSize(7, 16777215))
        self.lSlash.setObjectName("lSlash")
        self.gridLayout_4.addWidget(self.lSlash, 3, 3, 1, 1)
        self.btnSearchNext = QtWidgets.QPushButton(self.scrollAreaWidgetContents_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnSearchNext.sizePolicy().hasHeightForWidth())
        self.btnSearchNext.setSizePolicy(sizePolicy)
        self.btnSearchNext.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.btnSearchNext.setText("")
        icon = QtGui.QIcon.fromTheme("go-next")
        self.btnSearchNext.setIcon(icon)
        self.btnSearchNext.setObjectName("btnSearchNext")
        self.gridLayout_4.addWidget(self.btnSearchNext, 3, 5, 1, 1)
        self.chkBoxLockSIV = QtWidgets.QCheckBox(self.scrollAreaWidgetContents_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.chkBoxLockSIV.sizePolicy().hasHeightForWidth())
        self.chkBoxLockSIV.setSizePolicy(sizePolicy)
        self.chkBoxLockSIV.setObjectName("chkBoxLockSIV")
        self.gridLayout_4.addWidget(self.chkBoxLockSIV, 1, 0, 1, 5)
        self.scrollArea_5.setWidget(self.scrollAreaWidgetContents_5)
        self.gridLayout_7.addWidget(self.scrollArea_5, 9, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lSamplesInViewModulatedText = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSamplesInViewModulatedText.sizePolicy().hasHeightForWidth())
        self.lSamplesInViewModulatedText.setSizePolicy(sizePolicy)
        self.lSamplesInViewModulatedText.setObjectName("lSamplesInViewModulatedText")
        self.horizontalLayout.addWidget(self.lSamplesInViewModulatedText)
        self.lSamplesInViewModulated = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSamplesInViewModulated.sizePolicy().hasHeightForWidth())
        self.lSamplesInViewModulated.setSizePolicy(sizePolicy)
        self.lSamplesInViewModulated.setObjectName("lSamplesInViewModulated")
        self.horizontalLayout.addWidget(self.lSamplesInViewModulated)
        self.label_9 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout.addWidget(self.label_9)
        self.lModulatedSelectedSamples = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.lModulatedSelectedSamples.setObjectName("lModulatedSelectedSamples")
        self.horizontalLayout.addWidget(self.lModulatedSelectedSamples)
        self.gridLayout_7.addLayout(self.horizontalLayout, 6, 1, 1, 1)
        self.scrollArea_3 = QtWidgets.QScrollArea(self.scrollAreaWidgetContents_2)
        self.scrollArea_3.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea_3.setWidgetResizable(True)
        self.scrollArea_3.setObjectName("scrollArea_3")
        self.scrollAreaWidgetContents_3 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_3.setGeometry(QtCore.QRect(0, 0, 373, 141))
        self.scrollAreaWidgetContents_3.setObjectName("scrollAreaWidgetContents_3")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents_3)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.spinBoxSampleRate = KillerDoubleSpinBox(self.scrollAreaWidgetContents_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinBoxSampleRate.sizePolicy().hasHeightForWidth())
        self.spinBoxSampleRate.setSizePolicy(sizePolicy)
        self.spinBoxSampleRate.setDecimals(10)
        self.spinBoxSampleRate.setMinimum(0.001)
        self.spinBoxSampleRate.setMaximum(999999999.0)
        self.spinBoxSampleRate.setObjectName("spinBoxSampleRate")
        self.gridLayout_2.addWidget(self.spinBoxSampleRate, 2, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem3, 3, 0, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.scrollAreaWidgetContents_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 2, 0, 1, 1)
        self.label = QtWidgets.QLabel(self.scrollAreaWidgetContents_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 1, 0, 1, 1)
        self.spinBoxSamplesPerSymbol = QtWidgets.QSpinBox(self.scrollAreaWidgetContents_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinBoxSamplesPerSymbol.sizePolicy().hasHeightForWidth())
        self.spinBoxSamplesPerSymbol.setSizePolicy(sizePolicy)
        self.spinBoxSamplesPerSymbol.setMinimum(1)
        self.spinBoxSamplesPerSymbol.setMaximum(999999)
        self.spinBoxSamplesPerSymbol.setObjectName("spinBoxSamplesPerSymbol")
        self.gridLayout_2.addWidget(self.spinBoxSamplesPerSymbol, 1, 1, 1, 1)
        self.linEdDataBits = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.linEdDataBits.sizePolicy().hasHeightForWidth())
        self.linEdDataBits.setSizePolicy(sizePolicy)
        self.linEdDataBits.setObjectName("linEdDataBits")
        self.gridLayout_2.addWidget(self.linEdDataBits, 0, 0, 1, 2)
        self.scrollArea_3.setWidget(self.scrollAreaWidgetContents_3)
        self.gridLayout_7.addWidget(self.scrollArea_3, 3, 0, 1, 1)
        self.scrollArea_2 = QtWidgets.QScrollArea(self.scrollAreaWidgetContents_2)
        self.scrollArea_2.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea_2.setWidgetResizable(True)
        self.scrollArea_2.setObjectName("scrollArea_2")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 353, 143))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout.setObjectName("gridLayout")
        self.lCarrierFreq = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lCarrierFreq.sizePolicy().hasHeightForWidth())
        self.lCarrierFreq.setSizePolicy(sizePolicy)
        self.lCarrierFreq.setObjectName("lCarrierFreq")
        self.gridLayout.addWidget(self.lCarrierFreq, 0, 0, 1, 1)
        self.doubleSpinBoxCarrierFreq = KillerDoubleSpinBox(self.scrollAreaWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.doubleSpinBoxCarrierFreq.sizePolicy().hasHeightForWidth())
        self.doubleSpinBoxCarrierFreq.setSizePolicy(sizePolicy)
        self.doubleSpinBoxCarrierFreq.setSuffix("")
        self.doubleSpinBoxCarrierFreq.setDecimals(10)
        self.doubleSpinBoxCarrierFreq.setMinimum(0.0)
        self.doubleSpinBoxCarrierFreq.setMaximum(99999999999.0)
        self.doubleSpinBoxCarrierFreq.setObjectName("doubleSpinBoxCarrierFreq")
        self.gridLayout.addWidget(self.doubleSpinBoxCarrierFreq, 0, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.doubleSpinBoxCarrierPhase = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.doubleSpinBoxCarrierPhase.sizePolicy().hasHeightForWidth())
        self.doubleSpinBoxCarrierPhase.setSizePolicy(sizePolicy)
        self.doubleSpinBoxCarrierPhase.setDecimals(3)
        self.doubleSpinBoxCarrierPhase.setMaximum(360.0)
        self.doubleSpinBoxCarrierPhase.setObjectName("doubleSpinBoxCarrierPhase")
        self.gridLayout.addWidget(self.doubleSpinBoxCarrierPhase, 1, 1, 1, 1)
        self.btnAutoDetect = QtWidgets.QPushButton(self.scrollAreaWidgetContents)
        self.btnAutoDetect.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnAutoDetect.sizePolicy().hasHeightForWidth())
        self.btnAutoDetect.setSizePolicy(sizePolicy)
        self.btnAutoDetect.setObjectName("btnAutoDetect")
        self.gridLayout.addWidget(self.btnAutoDetect, 2, 0, 1, 2)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem4, 3, 0, 1, 1)
        self.scrollArea_2.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout_7.addWidget(self.scrollArea_2, 1, 0, 1, 1)
        self.lPlus = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lPlus.sizePolicy().hasHeightForWidth())
        self.lPlus.setSizePolicy(sizePolicy)
        self.lPlus.setMaximumSize(QtCore.QSize(32, 32))
        self.lPlus.setText("")
        self.lPlus.setPixmap(QtGui.QPixmap(":/icons/icons/plus.svg"))
        self.lPlus.setScaledContents(True)
        self.lPlus.setAlignment(QtCore.Qt.AlignCenter)
        self.lPlus.setObjectName("lPlus")
        self.gridLayout_7.addWidget(self.lPlus, 2, 2, 1, 1)
        self.gVCarrier = ZoomableGraphicView(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.gVCarrier.sizePolicy().hasHeightForWidth())
        self.gVCarrier.setSizePolicy(sizePolicy)
        self.gVCarrier.setAcceptDrops(False)
        self.gVCarrier.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.gVCarrier.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.gVCarrier.setRenderHints(QtGui.QPainter.Antialiasing|QtGui.QPainter.HighQualityAntialiasing)
        self.gVCarrier.setDragMode(QtWidgets.QGraphicsView.NoDrag)
        self.gVCarrier.setObjectName("gVCarrier")
        self.gridLayout_7.addWidget(self.gVCarrier, 1, 1, 1, 3)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem5, 2, 1, 1, 1)
        self.gVModulated = ZoomableGraphicView(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.gVModulated.sizePolicy().hasHeightForWidth())
        self.gVModulated.setSizePolicy(sizePolicy)
        self.gVModulated.setAcceptDrops(False)
        self.gVModulated.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.gVModulated.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.gVModulated.setRenderHints(QtGui.QPainter.Antialiasing|QtGui.QPainter.HighQualityAntialiasing)
        self.gVModulated.setDragMode(QtWidgets.QGraphicsView.NoDrag)
        self.gVModulated.setObjectName("gVModulated")
        self.gridLayout_7.addWidget(self.gVModulated, 5, 1, 1, 3)
        self.gVData = ZoomableGraphicView(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.gVData.sizePolicy().hasHeightForWidth())
        self.gVData.setSizePolicy(sizePolicy)
        self.gVData.setAcceptDrops(False)
        self.gVData.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.gVData.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.gVData.setRenderHints(QtGui.QPainter.Antialiasing|QtGui.QPainter.HighQualityAntialiasing)
        self.gVData.setDragMode(QtWidgets.QGraphicsView.NoDrag)
        self.gVData.setObjectName("gVData")
        self.gridLayout_7.addWidget(self.gVData, 3, 1, 1, 3)
        self.scrollArea_4 = QtWidgets.QScrollArea(self.scrollAreaWidgetContents_2)
        self.scrollArea_4.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea_4.setWidgetResizable(True)
        self.scrollArea_4.setObjectName("scrollArea_4")
        self.scrollAreaWidgetContents_4 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_4.setGeometry(QtCore.QRect(0, 0, 353, 227))
        self.scrollAreaWidgetContents_4.setObjectName("scrollAreaWidgetContents_4")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents_4)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.spinBoxBitsPerSymbol = QtWidgets.QSpinBox(self.scrollAreaWidgetContents_4)
        self.spinBoxBitsPerSymbol.setMinimum(1)
        self.spinBoxBitsPerSymbol.setMaximum(10)
        self.spinBoxBitsPerSymbol.setObjectName("spinBoxBitsPerSymbol")
        self.gridLayout_3.addWidget(self.spinBoxBitsPerSymbol, 1, 1, 1, 1)
        self.spinBoxGaussBT = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents_4)
        self.spinBoxGaussBT.setMinimum(0.01)
        self.spinBoxGaussBT.setMaximum(0.99)
        self.spinBoxGaussBT.setSingleStep(0.01)
        self.spinBoxGaussBT.setObjectName("spinBoxGaussBT")
        self.gridLayout_3.addWidget(self.spinBoxGaussBT, 3, 1, 1, 1)
        self.lGaussWidth = QtWidgets.QLabel(self.scrollAreaWidgetContents_4)
        self.lGaussWidth.setObjectName("lGaussWidth")
        self.gridLayout_3.addWidget(self.lGaussWidth, 4, 0, 1, 1)
        self.lGaussBT = QtWidgets.QLabel(self.scrollAreaWidgetContents_4)
        self.lGaussBT.setObjectName("lGaussBT")
        self.gridLayout_3.addWidget(self.lGaussBT, 3, 0, 1, 1)
        self.spinBoxGaussFilterWidth = QtWidgets.QDoubleSpinBox(self.scrollAreaWidgetContents_4)
        self.spinBoxGaussFilterWidth.setMinimum(0.01)
        self.spinBoxGaussFilterWidth.setMaximum(100.0)
        self.spinBoxGaussFilterWidth.setSingleStep(0.01)
        self.spinBoxGaussFilterWidth.setProperty("value", 1.0)
        self.spinBoxGaussFilterWidth.setObjectName("spinBoxGaussFilterWidth")
        self.gridLayout_3.addWidget(self.spinBoxGaussFilterWidth, 4, 1, 1, 1)
        self.labelBitsPerSymbol = QtWidgets.QLabel(self.scrollAreaWidgetContents_4)
        self.labelBitsPerSymbol.setObjectName("labelBitsPerSymbol")
        self.gridLayout_3.addWidget(self.labelBitsPerSymbol, 1, 0, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem6, 5, 0, 1, 1)
        spacerItem7 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem7, 5, 1, 1, 1)
        self.lineEditParameters = QtWidgets.QLineEdit(self.scrollAreaWidgetContents_4)
        self.lineEditParameters.setClearButtonEnabled(False)
        self.lineEditParameters.setObjectName("lineEditParameters")
        self.gridLayout_3.addWidget(self.lineEditParameters, 2, 1, 1, 1)
        self.comboBoxModulationType = QtWidgets.QComboBox(self.scrollAreaWidgetContents_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxModulationType.sizePolicy().hasHeightForWidth())
        self.comboBoxModulationType.setSizePolicy(sizePolicy)
        self.comboBoxModulationType.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.comboBoxModulationType.setObjectName("comboBoxModulationType")
        self.comboBoxModulationType.addItem("")
        self.comboBoxModulationType.addItem("")
        self.comboBoxModulationType.addItem("")
        self.comboBoxModulationType.addItem("")
        self.gridLayout_3.addWidget(self.comboBoxModulationType, 0, 0, 1, 2)
        self.labelParameters = QtWidgets.QLabel(self.scrollAreaWidgetContents_4)
        self.labelParameters.setObjectName("labelParameters")
        self.gridLayout_3.addWidget(self.labelParameters, 2, 0, 1, 1)
        self.scrollArea_4.setWidget(self.scrollAreaWidgetContents_4)
        self.gridLayout_7.addWidget(self.scrollArea_4, 5, 0, 1, 1)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem8, 4, 3, 1, 1)
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem9, 8, 3, 1, 1)
        self.lEqual_qm = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lEqual_qm.sizePolicy().hasHeightForWidth())
        self.lEqual_qm.setSizePolicy(sizePolicy)
        self.lEqual_qm.setMaximumSize(QtCore.QSize(32, 32))
        self.lEqual_qm.setText("")
        self.lEqual_qm.setPixmap(QtGui.QPixmap(":/icons/icons/equals_qm.svg"))
        self.lEqual_qm.setScaledContents(True)
        self.lEqual_qm.setAlignment(QtCore.Qt.AlignCenter)
        self.lEqual_qm.setObjectName("lEqual_qm")
        self.gridLayout_7.addWidget(self.lEqual_qm, 8, 2, 1, 1)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lSamplesInViewOrigSignalText = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSamplesInViewOrigSignalText.sizePolicy().hasHeightForWidth())
        self.lSamplesInViewOrigSignalText.setSizePolicy(sizePolicy)
        self.lSamplesInViewOrigSignalText.setObjectName("lSamplesInViewOrigSignalText")
        self.horizontalLayout_2.addWidget(self.lSamplesInViewOrigSignalText)
        self.lSamplesInViewOrigSignal = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lSamplesInViewOrigSignal.sizePolicy().hasHeightForWidth())
        self.lSamplesInViewOrigSignal.setSizePolicy(sizePolicy)
        self.lSamplesInViewOrigSignal.setObjectName("lSamplesInViewOrigSignal")
        self.horizontalLayout_2.addWidget(self.lSamplesInViewOrigSignal)
        self.label_10 = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_2.addWidget(self.label_10)
        self.lOriginalSignalSamplesSelected = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.lOriginalSignalSamplesSelected.setObjectName("lOriginalSignalSamplesSelected")
        self.horizontalLayout_2.addWidget(self.lOriginalSignalSamplesSelected)
        self.gridLayout_7.addLayout(self.horizontalLayout_2, 10, 1, 1, 1)
        self.gridLayout_7.setRowStretch(1, 1)
        self.gridLayout_7.setRowStretch(3, 1)
        self.gridLayout_7.setRowStretch(5, 1)
        self.gridLayout_7.setRowStretch(8, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents_2)
        self.verticalLayout.addWidget(self.scrollArea)

        self.retranslateUi(DialogModulation)
        DialogModulation.setTabOrder(self.btnAddModulation, self.scrollArea_2)
        DialogModulation.setTabOrder(self.scrollArea_2, self.doubleSpinBoxCarrierFreq)
        DialogModulation.setTabOrder(self.doubleSpinBoxCarrierFreq, self.doubleSpinBoxCarrierPhase)
        DialogModulation.setTabOrder(self.doubleSpinBoxCarrierPhase, self.btnAutoDetect)
        DialogModulation.setTabOrder(self.btnAutoDetect, self.scrollArea_3)
        DialogModulation.setTabOrder(self.scrollArea_3, self.linEdDataBits)
        DialogModulation.setTabOrder(self.linEdDataBits, self.spinBoxSamplesPerSymbol)
        DialogModulation.setTabOrder(self.spinBoxSamplesPerSymbol, self.spinBoxSampleRate)
        DialogModulation.setTabOrder(self.spinBoxSampleRate, self.scrollArea_4)
        DialogModulation.setTabOrder(self.scrollArea_4, self.comboBoxModulationType)
        DialogModulation.setTabOrder(self.comboBoxModulationType, self.spinBoxBitsPerSymbol)
        DialogModulation.setTabOrder(self.spinBoxBitsPerSymbol, self.lineEditParameters)
        DialogModulation.setTabOrder(self.lineEditParameters, self.spinBoxGaussBT)
        DialogModulation.setTabOrder(self.spinBoxGaussBT, self.spinBoxGaussFilterWidth)
        DialogModulation.setTabOrder(self.spinBoxGaussFilterWidth, self.scrollArea_5)
        DialogModulation.setTabOrder(self.scrollArea_5, self.treeViewSignals)
        DialogModulation.setTabOrder(self.treeViewSignals, self.chkBoxLockSIV)
        DialogModulation.setTabOrder(self.chkBoxLockSIV, self.gVCarrier)
        DialogModulation.setTabOrder(self.gVCarrier, self.gVData)
        DialogModulation.setTabOrder(self.gVData, self.gVModulated)
        DialogModulation.setTabOrder(self.gVModulated, self.gVOriginalSignal)
        DialogModulation.setTabOrder(self.gVOriginalSignal, self.cbShowDataBitsOnly)
        DialogModulation.setTabOrder(self.cbShowDataBitsOnly, self.btnSearchPrev)
        DialogModulation.setTabOrder(self.btnSearchPrev, self.btnSearchNext)
        DialogModulation.setTabOrder(self.btnSearchNext, self.btnRemoveModulation)
        DialogModulation.setTabOrder(self.btnRemoveModulation, self.comboBoxCustomModulations)
        DialogModulation.setTabOrder(self.comboBoxCustomModulations, self.scrollArea)

    def retranslateUi(self, DialogModulation):
        _translate = QtCore.QCoreApplication.translate
        DialogModulation.setWindowTitle(_translate("DialogModulation", "Modulation"))
        self.comboBoxCustomModulations.setItemText(0, _translate("DialogModulation", "My Modulation"))
        self.btnAddModulation.setText(_translate("DialogModulation", "..."))
        self.btnRemoveModulation.setText(_translate("DialogModulation", "..."))
        self.label_5.setText(_translate("DialogModulation", "Data (raw bits)"))
        self.label_6.setText(_translate("DialogModulation", "Modulation"))
        self.label_7.setText(_translate("DialogModulation", "Original Signal (drag&drop)"))
        self.label_4.setText(_translate("DialogModulation", "Carrier"))
        self.lCurrentSearchResult.setText(_translate("DialogModulation", "-"))
        self.cbShowDataBitsOnly.setText(_translate("DialogModulation", "Show Only Data Sequence\n"
"(10)"))
        self.lTotalSearchresults.setText(_translate("DialogModulation", "-"))
        self.lSlash.setText(_translate("DialogModulation", "/"))
        self.chkBoxLockSIV.setText(_translate("DialogModulation", "Lock view to original signal"))
        self.lSamplesInViewModulatedText.setText(_translate("DialogModulation", "Samples in View:"))
        self.lSamplesInViewModulated.setToolTip(_translate("DialogModulation", "<html><head/><body><p>Shown Samples in View:</p><p><span style=\" font-weight:600; color:#ff0000;\">Red</span> - if samples in view differ from original signal</p><p><span style=\" font-weight:600;\">Normal</span> - if samples in view are equal to the original signal</p></body></html>"))
        self.lSamplesInViewModulated.setText(_translate("DialogModulation", "101010121"))
        self.label_9.setText(_translate("DialogModulation", "Samples selected:"))
        self.lModulatedSelectedSamples.setText(_translate("DialogModulation", "0"))
        self.label_3.setText(_translate("DialogModulation", "Sample Rate (Sps):"))
        self.label.setText(_translate("DialogModulation", "Samples per Symbol:"))
        self.linEdDataBits.setPlaceholderText(_translate("DialogModulation", "Enter Data Bits here"))
        self.lCarrierFreq.setText(_translate("DialogModulation", "Frequency:"))
        self.label_2.setText(_translate("DialogModulation", "Phase:"))
        self.doubleSpinBoxCarrierPhase.setSuffix(_translate("DialogModulation", "°"))
        self.btnAutoDetect.setToolTip(_translate("DialogModulation", "<html><head/><body><p>Auto detect the frequency based on the original signal. You have to select a signal (<span style=\" font-weight:600;\">bottom of this window</span>) to use this feature.</p><p><br/></p><p>Select a signal by dragging it from the tree and dropping it on the graphics pane to the right.</p></body></html>"))
        self.btnAutoDetect.setText(_translate("DialogModulation", "Auto detect from original signal"))
        self.lGaussWidth.setText(_translate("DialogModulation", "Gauss filter width:"))
        self.lGaussBT.setText(_translate("DialogModulation", "Gauss BT:"))
        self.labelBitsPerSymbol.setText(_translate("DialogModulation", "Bits per Symbol:"))
        self.comboBoxModulationType.setItemText(0, _translate("DialogModulation", "Amplitude Shift Keying (ASK)"))
        self.comboBoxModulationType.setItemText(1, _translate("DialogModulation", "Frequency Shift Keying (FSK)"))
        self.comboBoxModulationType.setItemText(2, _translate("DialogModulation", "Gaussian Frequency Shift Keying (GFSK)"))
        self.comboBoxModulationType.setItemText(3, _translate("DialogModulation", "Phase Shift Keying (PSK)"))
        self.labelParameters.setText(_translate("DialogModulation", "Parameters:"))
        self.lSamplesInViewOrigSignalText.setText(_translate("DialogModulation", "Samples in View:"))
        self.lSamplesInViewOrigSignal.setToolTip(_translate("DialogModulation", "<html><head/><body><p>Shown Samples in View:</p><p><span style=\" font-weight:600; color:#ff0000;\">Red</span> - if samples in view differ from original signal</p><p><span style=\" font-weight:600;\">Normal</span> - if samples in view are equal to the original signal</p></body></html>"))
        self.lSamplesInViewOrigSignal.setText(_translate("DialogModulation", "0"))
        self.label_10.setText(_translate("DialogModulation", "Samples selected:"))
        self.lOriginalSignalSamplesSelected.setText(_translate("DialogModulation", "0"))
from urh.ui.KillerDoubleSpinBox import KillerDoubleSpinBox
from urh.ui.views.ModulatorTreeView import ModulatorTreeView
from urh.ui.views.ZoomAndDropableGraphicView import ZoomAndDropableGraphicView
from urh.ui.views.ZoomableGraphicView import ZoomableGraphicView
from . import urh_rc



ctypedef fused iq:
    char
    unsigned char
    short
    unsigned short
    float

ctypedef iq[:, ::1] IQ

from libc.stdint cimport uint64_t, uint8_t, int64_t

cpdef uint64_t bit_array_to_number(uint8_t[:] bits, int64_t end, int64_t start=*) nogil


from urh.signalprocessing.Message import Message
from urh.signalprocessing.Participant import Participant


class Transcript(object):
    FORMAT = "{0} ({1}->{2}): {3}"

    def __init__(self):
        self.__data = []

    def append(self, source: Participant, destination: Participant, msg: Message, index: int):
        if len(self.__data) == 0:
            self.__data.append([])

        self.__data[-1].append((source, destination, msg, index))

    def start_new_round(self):
        if len(self.__data) == 0 or len(self.__data[-1]) > 0:
            self.__data.append([])

    def clear(self):
        self.__data.clear()

    def get_for_all_participants(self, all_rounds: bool, use_bit=True) -> list:
        result = []
        if len(self.__data) == 0:
            return result

        rng = range(0, len(self.__data)) if all_rounds else range(len(self.__data)-1, len(self.__data))

        for i in rng:
            for source, destination, msg, msg_index in self.__data[i]:
                data = msg.plain_bits_str if use_bit else msg.plain_hex_str
                result.append(self.FORMAT.format(msg_index, source.shortname, destination.shortname, data))

            if i != len(self.__data) - 1:
                result.append("")

        return result

    def get_for_participant(self, participant: Participant) -> str:
        if len(self.__data) == 0:
            return ""

        result = []
        for source, destination, msg, _ in self.__data[-1]:
            if participant == destination:
                result.append("->" + msg.plain_bits_str)
            elif participant == source:
                result.append("<-" + msg.plain_bits_str)

        return "\n".join(result)



import locale
import time

from PyQt5.QtCore import pyqtSlot, QTimer, pyqtSignal, Qt
from PyQt5.QtGui import QCloseEvent, QTransform
from PyQt5.QtWidgets import QDialog, QGraphicsView

from urh import settings
from urh.controller.widgets.DeviceSettingsWidget import DeviceSettingsWidget
from urh.dev.BackendHandler import BackendHandler, Backends
from urh.dev.VirtualDevice import VirtualDevice
from urh.plugins.NetworkSDRInterface.NetworkSDRInterfacePlugin import NetworkSDRInterfacePlugin
from urh.ui.ui_send_recv import Ui_SendRecvDialog
from urh.util import util
from urh.util.Errors import Errors
from urh.util.Formatter import Formatter
from urh.util.Logger import logger
from urh.util.ProjectManager import ProjectManager


class SendRecvDialog(QDialog):
    device_parameters_changed = pyqtSignal(dict)

    def __init__(self, project_manager: ProjectManager, is_tx: bool, continuous_send_mode=False, parent=None, testing_mode=False):
        super().__init__(parent)
        self.is_tx = is_tx
        self.update_interval = 25

        # This flag is needed. Will cause memory leak otherwise.
        self.setAttribute(Qt.WA_DeleteOnClose)

        self.setWindowFlags(Qt.Window)
        self.testing_mode = testing_mode

        self.ui = Ui_SendRecvDialog()
        self.ui.setupUi(self)
        util.set_splitter_stylesheet(self.ui.splitter)

        self.ui.txtEditErrors.setFont(util.get_monospace_font())

        self.graphics_view = None  # type: QGraphicsView

        self.backend_handler = BackendHandler()

        self.ui.btnStop.setEnabled(False)
        self.ui.btnSave.setEnabled(False)

        self.start = 0

        self.device_settings_widget = DeviceSettingsWidget(project_manager, is_tx,
                                                           backend_handler=self.backend_handler,
                                                           continuous_send_mode=continuous_send_mode)
        self.ui.scrollAreaWidgetContents_2.layout().insertWidget(0, self.device_settings_widget)

        if testing_mode:
            self.device_settings_widget.ui.cbDevice.setCurrentText(NetworkSDRInterfacePlugin.NETWORK_SDR_NAME)

        self.timer = QTimer(self)
        self.restoreGeometry(settings.read("{}/geometry".format(self.__class__.__name__), type=bytes))

        self.ui.splitter.setSizes([int(0.4 * self.width()), int(0.6 * self.width())])

        self.current_y_slider_value = 1

    @property
    def is_rx(self) -> bool:
        return not self.is_tx

    @property
    def has_empty_device_list(self):
        return self.device_settings_widget.ui.cbDevice.count() == 0

    @property
    def device(self) -> VirtualDevice:
        return self.device_settings_widget.device

    @device.setter
    def device(self, value):
        self.device_settings_widget.device = value

    @property
    def selected_device_name(self) -> str:
        return self.device_settings_widget.ui.cbDevice.currentText()

    def _eliminate_graphic_view(self):
        if self.graphics_view is not None:
            self.graphics_view.eliminate()

        self.graphics_view = None

    def hide_send_ui_items(self):
        for item in ("lblCurrentRepeatValue", "progressBarMessage",
                     "lblRepeatText", "lSamplesSentText", "progressBarSample", "labelCurrentMessage"):
            getattr(self.ui, item).hide()

    def hide_receive_ui_items(self):
        for item in ("lSamplesCaptured", "lSamplesCapturedText", "lSignalSize", "lSignalSizeText",
                     "lTime", "lTimeText", "btnSave", "labelReceiveBufferFull", "lReceiveBufferFullText"):
            getattr(self.ui, item).hide()

    def set_device_ui_items_enabled(self, enabled: bool):
        self.device_settings_widget.setEnabled(enabled)

    def create_connects(self):
        self.ui.btnStart.clicked.connect(self.on_start_clicked)
        self.ui.btnStop.clicked.connect(self.on_stop_clicked)
        self.ui.btnClear.clicked.connect(self.on_clear_clicked)

        self.timer.timeout.connect(self.update_view)
        self.ui.sliderYscale.valueChanged.connect(self.on_slider_y_scale_value_changed)

        self.device_settings_widget.selected_device_changed.connect(self.on_selected_device_changed)
        self.device_settings_widget.device_parameters_changed.connect(self.device_parameters_changed.emit)

    def _create_device_connects(self):
        self.device.stopped.connect(self.on_device_stopped)
        self.device.started.connect(self.on_device_started)
        self.device.sender_needs_restart.connect(self._restart_device_thread)

    def reset(self):
        self.device.current_index = 0
        self.device.current_iteration = 0
        self.ui.lSamplesCaptured.setText("0")
        self.ui.lSignalSize.setText("0")
        self.ui.lTime.setText("0")
        self.ui.lblCurrentRepeatValue.setText("-")
        self.ui.progressBarSample.setValue(0)
        self.ui.progressBarMessage.setValue(0)
        self.ui.btnSave.setEnabled(False)

    def init_device(self):
        pass

    def save_before_close(self):
        return True

    def emit_editing_finished_signals(self):
        self.device_settings_widget.emit_editing_finished_signals()

    @pyqtSlot()
    def on_selected_device_changed(self):
        if hasattr(self.scene_manager, "plot_data"):
            self.scene_manager.plot_data = None

        self.init_device()

        self.graphics_view.scene_manager = self.scene_manager
        self.graphics_view.setScene(self.scene_manager.scene)

    @pyqtSlot()
    def on_start_clicked(self):
        self.emit_editing_finished_signals()

    @pyqtSlot()
    def on_stop_clicked(self):
        self.device.stop("Stopped receiving: Stop button clicked")

    @pyqtSlot()
    def on_device_stopped(self):
        if self.graphics_view is not None:
            self.graphics_view.capturing_data = False
        self.set_device_ui_items_enabled(True)
        self.ui.btnStart.setEnabled(True)
        self.ui.btnStop.setEnabled(False)
        self.ui.btnSave.setEnabled(self.device.current_index > 0)
        self.device_settings_widget.ui.comboBoxDeviceIdentifier.setEnabled(True)
        self.device_settings_widget.ui.btnRefreshDeviceIdentifier.setEnabled(True)
        self.device_settings_widget.set_bandwidth_status()

        self.timer.stop()
        self.update_view()

    @pyqtSlot()
    def on_device_started(self):
        self.ui.txtEditErrors.clear()
        if self.graphics_view is not None:
            self.graphics_view.capturing_data = True
        self.ui.btnSave.setEnabled(False)
        self.ui.btnStart.setEnabled(False)
        self.ui.btnStop.setEnabled(True)
        self.device_settings_widget.ui.comboBoxDeviceIdentifier.setEnabled(False)
        self.device_settings_widget.ui.btnRefreshDeviceIdentifier.setEnabled(False)

        self.timer.start(self.update_interval)

    def __parse_error_messages(self, messages):
        messages = messages.lower()

        if "no devices found for" in messages:
            self.device.stop_on_error("Could not establish connection to USRP")
            Errors.usrp_found()
            self.on_clear_clicked()

        elif any(e in messages for e in ("hackrf_error_not_found", "hackrf_error_libusb")):
            self.device.stop_on_error("Could not establish connection to HackRF")
            Errors.hackrf_not_found()
            self.on_clear_clicked()

        elif "no module named gnuradio" in messages:
            self.device.stop_on_error("Did not find gnuradio.")
            Errors.gnuradio_not_installed()
            self.on_clear_clicked()

        elif "rtlsdr-open: error code: -1" in messages:
            self.device.stop_on_error("Could not open a RTL-SDR device.")
            self.on_clear_clicked()

        elif "rtlsdr-open: error code: -12" in messages:
            self.device.stop_on_error("Could not open a RTL-SDR device")
            Errors.rtlsdr_sdr_driver()
            self.on_clear_clicked()

        elif "Address already in use" in messages:
            self._restart_device_thread()

    def update_view(self):
        txt = self.ui.txtEditErrors.toPlainText()
        new_messages = self.device.read_messages()

        self.__parse_error_messages(new_messages)

        if len(new_messages) > 1:
            self.ui.txtEditErrors.setPlainText(txt + new_messages)

        self.ui.lSamplesCaptured.setText(Formatter.big_value_with_suffix(self.device.current_index, decimals=1))
        self.ui.lSignalSize.setText(locale.format_string("%.2f", (8 * self.device.current_index) / (1024 ** 2)))
        self.ui.lTime.setText(locale.format_string("%.2f", self.device.current_index / self.device.sample_rate))

        if self.is_rx and self.device.data is not None and len(self.device.data) > 0:
            self.ui.labelReceiveBufferFull.setText("{0}%".format(int(100 * self.device.current_index /
                                                                     len(self.device.data))))

        if self.device.current_index == 0:
            return False

        return True

    def _restart_device_thread(self):
        self.device.stop("Restarting with new port")

        if self.device.backend == Backends.grc:
            self.device.increase_gr_port()

        self.device.start()

    @pyqtSlot()
    def on_clear_clicked(self):
        pass

    def closeEvent(self, event: QCloseEvent):
        if self.device.backend is not Backends.none:
            self.emit_editing_finished_signals()

        self.timer.stop()

        self.device.stop("Dialog closed. Killing recording process.")
        logger.debug("Device stopped successfully.")

        if not self.testing_mode:
            if not self.save_before_close():
                event.ignore()
                return

        time.sleep(0.1)
        if self.device.backend not in (Backends.none, Backends.network):
            # Backend none is selected, when no device is available
            logger.debug("Cleaning up device")
            self.device.cleanup()
            logger.debug("Successfully cleaned up device")
            self.device_settings_widget.emit_device_parameters_changed()

        settings.write("{}/geometry".format(self.__class__.__name__), self.saveGeometry())

        if self.device is not None:
            self.device.free_data()

        self.scene_manager.eliminate()

        self._eliminate_graphic_view()

        super().closeEvent(event)

    @pyqtSlot(int)
    def on_slider_y_scale_value_changed(self, new_value: int):
        self.graphics_view.scale(1, new_value / self.current_y_slider_value)
        self.graphics_view.centerOn(0, 0)
        self.current_y_slider_value = new_value



# noinspection PyUnresolvedReferences
cimport numpy as np
import numpy as np
from PyQt5.QtCore import QByteArray, QDataStream
from PyQt5.QtGui import QPainterPath

# As we do not use any numpy C API functions we do no import_array here,
# because it can lead to OS X error: https://github.com/jopohl/urh/issues/273
# np.import_array()

from cython.parallel import prange
from urh.cythonext.util cimport iq

from urh import settings
import cython
import math
import struct

cpdef create_path(iq[:] samples, long long start, long long end, list subpath_ranges=None):
    cdef iq[:] values
    cdef long long[::1] sample_rng
    cdef np.int64_t[::1] x
    cdef iq sample, minimum, maximum, tmp
    cdef float scale_factor
    cdef long long i,j,index, chunk_end, num_samples, pixels_on_path, samples_per_pixel
    num_samples = end - start

    cdef dict type_lookup = {"char[:]": np.int8, "unsigned char[:]": np.uint8,
                             "short[:]": np.int16, "unsigned short[:]": np.uint16,
                             "float[:]": np.float32, "double[:]": np.float64}

    subpath_ranges = [(start, end)] if subpath_ranges is None else subpath_ranges
    pixels_on_path = settings.PIXELS_PER_PATH

    samples_per_pixel = <long long>(num_samples / pixels_on_path)

    cdef int num_threads = 0
    if samples_per_pixel < 20000:
        num_threads = 1

    if samples_per_pixel > 1:
        sample_rng = np.arange(start, end, samples_per_pixel, dtype=np.int64)
        values = np.zeros(2 * len(sample_rng), dtype=type_lookup[cython.typeof(samples)], order="C")
        scale_factor = num_samples / (2.0 * len(sample_rng))  # 2.0 is important to make it a float division!
        for i in prange(start, end, samples_per_pixel, nogil=True, schedule='static', num_threads=num_threads):
            chunk_end = i + samples_per_pixel
            if chunk_end >= end:
                chunk_end = end

            tmp = samples[i]
            minimum = tmp
            maximum = tmp

            for j in range(i + 1, chunk_end):
                sample = samples[j]
                if sample < minimum:
                    minimum = sample
                elif sample > maximum:
                    maximum = sample

            index = <long long>(2*(i-start)/samples_per_pixel)
            values[index] = minimum
            values[index + 1] = maximum

        x = np.repeat(sample_rng, 2)
    else:
        x = np.arange(start, end, dtype=np.int64)
        values = samples[start:end]
        scale_factor = 1.0

    cdef list result = []
    if scale_factor == 0:
        scale_factor = 1  # prevent division by zero

    for subpath_range in subpath_ranges:
        sub_start = ((((subpath_range[0]-start)/scale_factor) * scale_factor) - 2*scale_factor) / scale_factor
        sub_start =int(max(0, math.floor(sub_start)))
        sub_end = ((((subpath_range[1]-start)/scale_factor) * scale_factor) + 2*scale_factor) / scale_factor
        sub_end = int(max(0, math.ceil(sub_end)))
        result.append(array_to_QPath(x[sub_start:sub_end], values[sub_start:sub_end]))

    return result


cpdef create_live_path(iq[:] samples, unsigned int start, unsigned int end):
    return array_to_QPath(np.arange(start, end).astype(np.int64), samples)

cpdef array_to_QPath(np.int64_t[:] x, y):
    """
    Convert an array of x,y coordinates to QPainterPath as efficiently as possible.

    Speed this up using >> operator
    Format is:
        numVerts(i4)   0(i4)
        x(f8)   y(f8)   0(i4)    <-- 0 means this vertex does not connect
        x(f8)   y(f8)   1(i4)    <-- 1 means this vertex connects to the previous vertex
        ...
        0(i4)

     All values are big endian--pack using struct.pack('>d') or struct.pack('>i')
    """
    cdef long long n = x.shape[0]
    arr = np.zeros(n + 2, dtype=[('x', '>f8'), ('y', '>f8'), ('c', '>i4')])

    byte_view = arr.view(dtype=np.uint8)
    byte_view[:12] = 0
    byte_view.data[12:20] = struct.pack('>ii', n, 0)

    arr[1:n+1]['x'] = x
    arr[1:n+1]['y'] = np.negative(y)  # negate y since coordinate system is inverted
    arr[1:n+1]['c'] = 1

    cdef long long last_index = 20 * (n + 1)
    byte_view.data[last_index:last_index + 4] = struct.pack('>i', 0)

    try:
        buf = QByteArray.fromRawData(byte_view.data[12:last_index + 4])
    except TypeError:
        buf = QByteArray(byte_view.data[12:last_index + 4])

    path = QPainterPath()
    ds = QDataStream(buf)
    ds >> path

    return path



import copy
import random
import uuid
import xml.etree.ElementTree as ET

from PyQt5.QtCore import Qt

from urh import settings
from urh.signalprocessing.ChecksumLabel import ChecksumLabel
from urh.signalprocessing.FieldType import FieldType
from urh.signalprocessing.ProtocoLabel import ProtocolLabel
from urh.signalprocessing.Ruleset import Ruleset
from urh.util.Logger import logger


class MessageType(list):
    """
    A message type is a list of protocol fields.

    """

    __slots__ = ["name", "show", "__id", "assigned_by_ruleset", "ruleset", "assigned_by_logic_analyzer"]

    def __init__(self, name: str, iterable=None, id=None, ruleset=None):
        iterable = iterable if iterable else []
        super().__init__(iterable)

        self.name = name
        self.show = Qt.Checked
        self.__id = str(uuid.uuid4()) if id is None else id

        self.assigned_by_logic_analyzer = False
        self.assigned_by_ruleset = False
        self.ruleset = Ruleset() if ruleset is None else ruleset

    def __hash__(self):
        return hash(super)

    def __getitem__(self, index) -> ProtocolLabel:
        return super().__getitem__(index)

    def __repr__(self):
        return self.name + " " + super().__repr__()

    def __eq__(self, other):
        if isinstance(other, MessageType):
            return self.id == other.id
        else:
            return super().__eq__(other)

    @property
    def assign_manually(self):
        return not self.assigned_by_ruleset

    @property
    def id(self) -> str:
        return self.__id

    @property
    def checksum_labels(self) -> list:
        return [lbl for lbl in self if isinstance(lbl, ChecksumLabel)]

    @property
    def unlabeled_ranges(self):
        """

        :rtype: list[(int,int)]
        """
        return self.__get_unlabeled_ranges_from_labels(self)

    def __create_label(self, name: str, start: int, end: int, color_index: int, auto_created: bool,
                       field_type: FieldType):
        if field_type is not None:
            if field_type.function == FieldType.Function.CHECKSUM:
                # If we have sync or preamble labels start behind last one:
                pre_sync_label_ends = [lbl.end for lbl in self if lbl.is_preamble or lbl.is_sync]
                if len(pre_sync_label_ends) > 0:
                    range_start = max(pre_sync_label_ends)
                else:
                    range_start = 0

                if range_start >= start:
                    range_start = 0

                return ChecksumLabel(name=name, start=start, end=end, color_index=color_index, field_type=field_type,
                                     auto_created=auto_created, data_range_start=range_start)

        return ProtocolLabel(name=name, start=start, end=end, color_index=color_index, field_type=field_type,
                             auto_created=auto_created)

    @staticmethod
    def __get_unlabeled_ranges_from_labels(labels):
        """

        :type labels: list of ProtocolLabel
        :rtype: list[(int,int)]
        """
        start = 0
        result = []
        for lbl in labels:
            if lbl.start > start:
                result.append((start, lbl.start))
            start = lbl.end
        result.append((start, None))
        return result

    def unlabeled_ranges_with_other_mt(self, other_message_type):
        """

        :type other_message_type: MessageType
        :rtype: list[(int,int)]
        """
        labels = self + other_message_type
        labels.sort()
        return self.__get_unlabeled_ranges_from_labels(labels)

    def get_first_label_with_type(self, field_type: FieldType.Function) -> ProtocolLabel:
        return next((lbl for lbl in self if lbl.field_type and lbl.field_type.function == field_type), None)

    def num_labels_with_type(self, field_type: FieldType.Function) -> int:
        return len([lbl for lbl in self if lbl.field_type and lbl.field_type.function == field_type])

    def append(self, lbl: ProtocolLabel):
        super().append(lbl)
        self.sort()

    def give_new_id(self):
        self.__id = str(uuid.uuid4())

    def add_protocol_label(self, start: int, end: int, name=None, color_ind=None,
                           auto_created=False, type: FieldType = None) -> ProtocolLabel:

        name = "" if not name else name
        used_colors = [p.color_index for p in self]
        avail_colors = [i for i, _ in enumerate(settings.LABEL_COLORS) if i not in used_colors]

        if color_ind is None:
            if len(avail_colors) > 0:
                color_ind = avail_colors[0]
            else:
                color_ind = random.randint(0, len(settings.LABEL_COLORS) - 1)

        proto_label = self.__create_label(name=name, start=start, end=end, color_index=color_ind,
                                          auto_created=auto_created, field_type=type)

        if proto_label not in self:
            self.append(proto_label)
            self.sort()

        return proto_label  # Return label to set editor focus after adding

    def add_protocol_label_start_length(self, start: int, length: int, name=None, color_ind=None,
                                        auto_created=False, type: FieldType = None) -> ProtocolLabel:
        return self.add_protocol_label(start, start + length - 1, name, color_ind, auto_created, type)

    def add_label(self, lbl: ProtocolLabel, allow_overlapping=True):
        if allow_overlapping or not any(lbl.overlaps_with(l) for l in self):
            added = self.add_protocol_label(lbl.start, lbl.end - 1,
                                            name=lbl.name, color_ind=lbl.color_index,
                                            type=lbl.field_type)
            added.display_format_index = lbl.display_format_index
            added.display_bit_order_index = lbl.display_bit_order_index
            if isinstance(lbl, ChecksumLabel) and isinstance(added, ChecksumLabel):
                added.data_ranges = copy.copy(lbl.data_ranges)
                added.category = copy.copy(lbl.category)
                added.checksum = copy.copy(lbl.checksum)

    def remove(self, lbl: ProtocolLabel):
        if lbl in self:
            super().remove(lbl)
        else:
            logger.warning(lbl.name + " is not in set, so can't be removed")

    def change_field_type_of_label(self, label: ProtocolLabel, field_type: FieldType):
        if not isinstance(label, ProtocolLabel) and hasattr(label, "field_type"):
            # In case of SimulatorProtocolLabel
            label.field_type = field_type
            return

        is_crc_type = field_type is not None and field_type.function == FieldType.Function.CHECKSUM
        if is_crc_type != isinstance(label, ChecksumLabel):
            self[self.index(label)] = self.__create_label(label.name, label.start, label.end - 1,
                                                          label.color_index, label.auto_created, field_type)
        else:
            label.field_type = field_type

    def to_xml(self) -> ET.Element:
        result = ET.Element("message_type", attrib={"name": self.name, "id": self.id,
                                                    "assigned_by_ruleset": "1" if self.assigned_by_ruleset else "0",
                                                    "assigned_by_logic_analyzer": "1" if self.assigned_by_logic_analyzer else "0"})
        for lbl in self:
            try:
                result.append(lbl.to_xml())
            except TypeError:
                logger.error("Could not save label: " + str(lbl))

        result.append(self.ruleset.to_xml())

        return result

    @staticmethod
    def from_xml(tag: ET.Element):
        field_types_by_caption = {ft.caption: ft for ft in FieldType.load_from_xml()}

        name = tag.get("name", "blank")
        id = tag.get("id", None)
        assigned_by_ruleset = bool(int(tag.get("assigned_by_ruleset", 0)))
        assigned_by_logic_analyzer = bool(int(tag.get("assigned_by_logic_analyzer", 0)))
        labels = []
        for lbl_tag in tag.findall("label"):
            labels.append(ProtocolLabel.from_xml(lbl_tag, field_types_by_caption=field_types_by_caption))
        for lbl_tag in tag.findall("checksum_label"):
            labels.append(ChecksumLabel.from_xml(lbl_tag, field_types_by_caption=field_types_by_caption))
        result = MessageType(name=name, iterable=labels, id=id, ruleset=Ruleset.from_xml(tag.find("ruleset")))
        result.assigned_by_ruleset = assigned_by_ruleset
        result.assigned_by_logic_analyzer = assigned_by_logic_analyzer

        return result



# -*- coding: utf-8 -*-

#
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogCosta(object):
    def setupUi(self, DialogCosta):
        DialogCosta.setObjectName("DialogCosta")
        DialogCosta.resize(400, 300)
        self.verticalLayout = QtWidgets.QVBoxLayout(DialogCosta)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(DialogCosta)
        self.label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.labelLoopBandwidth = QtWidgets.QLabel(DialogCosta)
        self.labelLoopBandwidth.setObjectName("labelLoopBandwidth")
        self.horizontalLayout.addWidget(self.labelLoopBandwidth)
        self.doubleSpinBoxLoopBandwidth = QtWidgets.QDoubleSpinBox(DialogCosta)
        self.doubleSpinBoxLoopBandwidth.setDecimals(4)
        self.doubleSpinBoxLoopBandwidth.setObjectName("doubleSpinBoxLoopBandwidth")
        self.horizontalLayout.addWidget(self.doubleSpinBoxLoopBandwidth)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(DialogCosta)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)

        self.retranslateUi(DialogCosta)
        self.buttonBox.accepted.connect(DialogCosta.accept)
        self.buttonBox.rejected.connect(DialogCosta.reject)

    def retranslateUi(self, DialogCosta):
        _translate = QtCore.QCoreApplication.translate
        DialogCosta.setWindowTitle(_translate("DialogCosta", "Configure Costas Loop"))
        self.label.setText(_translate("DialogCosta", "URH uses a Costas loop for PSK demodulation. Configure the loop bandwidth below."))
        self.labelLoopBandwidth.setText(_translate("DialogCosta", "Loop Bandwidth:"))



from PyQt5.QtWidgets import QGraphicsTextItem
from PyQt5.QtGui import QFontDatabase

class UnlabeledRangeItem(QGraphicsTextItem):
    def __init__(self, parent):
        super().__init__(parent)

        font = QFontDatabase.systemFont(QFontDatabase.FixedFont)
        font.setPointSize(8)
        self.setFont(font)
        self.setPlainText("...")



from urh.dev.native.lib.cbladerf cimport *
from libc.stdint cimport int16_t
from libcpp cimport bool
from libc.stdlib cimport malloc, free

cdef bladerf* _c_device

cdef int CHANNEL = 0
cdef bool IS_TX = False

cpdef set_tx(bool is_tx):
    global IS_TX
    IS_TX = <bool>is_tx

cpdef bool get_tx():
    return IS_TX

cpdef set_channel(int channel):
    global CHANNEL
    CHANNEL = <int>channel
    return 0

cpdef int get_channel():
    return CHANNEL

IF BLADERF_API_VERSION >= 1.91:
    cpdef int get_current_bladerf_channel():
        if IS_TX:
            return BLADERF_CHANNEL_TX(get_channel())
        else:
            return BLADERF_CHANNEL_RX(get_channel())
ELSE:
    cpdef bladerf_module get_current_bladerf_channel():
        if IS_TX:
            return BLADERF_MODULE_RX
        else:
            return BLADERF_MODULE_TX

IF BLADERF_API_VERSION >= 1.91:
    cpdef bladerf_channel_layout get_current_channel_layout():
        if get_channel() == 0:
            if IS_TX:
                return BLADERF_TX_X1
            else:
                return BLADERF_RX_X1
        else:
            if IS_TX:
                return BLADERF_TX_X2
            else:
                return BLADERF_RX_X2
ELSE:
    cpdef bladerf_module get_current_channel_layout():
        return get_current_bladerf_channel()

cpdef int enable_module():
    return bladerf_enable_module(_c_device, get_current_bladerf_channel(), True)

cpdef int disable_module():
    return bladerf_enable_module(_c_device, get_current_bladerf_channel(), False)

cpdef list get_device_list():
    cdef bladerf_devinfo* dev_list
    cdef int i, num_devices
    num_devices = bladerf_get_device_list(&dev_list)

    if num_devices <= 0:
        return []

    result = []
    for i in range(0, num_devices):
        result.append(dev_list[i].serial.decode('utf-8'))

    bladerf_free_device_list(dev_list)
    return result

cpdef int open(str serial=""):
    cdef char* arg
    if serial == "":
        # Open first available
        return bladerf_open(&_c_device, NULL)
    else:
        argument_str = "*:serial={}".format(serial).encode('UTF-8')
        arg = <char*> argument_str
        return bladerf_open(&_c_device, arg)

cpdef void close():
    # disable the module when done, otherwise some warnings when closing during RX/TX
    disable_module()
    bladerf_close(_c_device)

cpdef int set_gain(bladerf_gain gain):
    if not IS_TX:
        # set to manual mode in rx case
        set_gain_mode_to_manual()

    return bladerf_set_gain(_c_device, get_current_bladerf_channel(), gain)

cpdef int set_gain_mode_to_manual():
    IF BLADERF_API_VERSION >= 1.91:
        bladerf_set_gain_mode(_c_device, get_current_bladerf_channel(), BLADERF_GAIN_MGC)
    ELSE:
        bladerf_set_gain_mode(_c_device, get_current_bladerf_channel(), BLADERF_GAIN_MANUAL)

cpdef int set_sample_rate(bladerf_sample_rate sample_rate):
    return bladerf_set_sample_rate(_c_device, get_current_bladerf_channel(), sample_rate, NULL)

cpdef bladerf_sample_rate get_sample_rate():
    cdef bladerf_sample_rate result = 0
    err = bladerf_get_sample_rate(_c_device, get_current_bladerf_channel(), &result)
    if err != 0:
        return 0

    return result

cpdef int set_bandwidth(bladerf_bandwidth bandwidth):
    return bladerf_set_bandwidth(_c_device, get_current_bladerf_channel(), bandwidth, NULL)

cpdef bladerf_bandwidth get_bandwidth():
    cdef bladerf_bandwidth result = 0
    err = bladerf_get_bandwidth(_c_device, get_current_bladerf_channel(), &result)

    if err != 0:
        return 0

    return result

cpdef int set_center_freq(bladerf_frequency frequency):
    return bladerf_set_frequency(_c_device, get_current_bladerf_channel(), frequency)

cpdef bladerf_frequency get_center_freq():
  cdef bladerf_frequency result = 0
  err = bladerf_get_frequency(_c_device, get_current_bladerf_channel(), &result)

  if err != 0:
      return 0

  return result

cpdef int set_bias_tee(on_or_off):
  IF BLADERF_API_VERSION >= 2:
    cdef bool bias_tee = 1 if on_or_off else 0
    return bladerf_set_bias_tee(_c_device, get_current_bladerf_channel(), bias_tee)
  ELSE:
    return -1

cpdef int get_bias_tee():
  IF BLADERF_API_VERSION >= 2:
    cdef bool result = 0
    err = bladerf_get_bias_tee(_c_device, get_current_bladerf_channel(), &result)
    if err < 0:
      return err

    return result
  ELSE:
    return -1


cpdef int prepare_sync():
    enable_module()
    return bladerf_sync_config(_c_device, get_current_channel_layout(), BLADERF_FORMAT_SC16_Q11, 32, 2048, 16, 100)

cpdef int16_t[:] receive_sync(connection, unsigned int num_samples):
    cdef int16_t *samples = <int16_t *> malloc(2*num_samples * sizeof(int16_t))
    if not samples:
        raise MemoryError()

    try:
        bladerf_sync_rx(_c_device, <void *>samples, num_samples, NULL, 100)
        return connection.send_bytes(<int16_t [:2*num_samples]>samples)
    finally:
        free(samples)

cpdef int send_sync(int16_t[::1] samples):
    cdef unsigned int num_samples = len(samples) // 2
    return bladerf_sync_tx(_c_device, &samples[0], num_samples, NULL, 100)

cpdef float get_api_version():
    cdef s_bladerf_version result

    bladerf_version(&result)

    print(result.major)
    print(result.minor)
    print(result.patch)

    print(result.describe.decode())
